//
//  rnn_entropy.cpp
//  Rnn
//
//  Created by Alessandro Sordoni on 3/10/2014.
//  Copyright (c) 2014 Alessandro Sordoni. All rights reserved.
//

#include <iostream>
#include <vector>
#include <unordered_map>
#include <string>
#include <algorithm>
#include <iomanip>
#include <iterator>
#include <thread>
#include <sstream>
#include <thread>
#include <mutex>

#include "FcgmTrainer.hpp"

int main(int argc, char* argv[])
{
	int threads = 1;
	std::string id = "unknown";
	std::string embeddings = "";
	int mb = 100;
	int epochs = 10;
	std::string output_path = "";
	std::string dataset_path = "";
	std::srand(1);

	if (argc == 1) {
		// Print help here!
		exit(1);
	}

	try {
		Config config = Config::ReadCommandLine(argc, argv);
			
		// Work queue
		Vocabulary vocabulary;
		RandomGenerator random;

		std::string datasetPath = config.GetString("dataset");
		std::string trainDataset = datasetPath + ".train";
		std::string validDataset = datasetPath + ".valid";
		std::string testDataset = datasetPath + ".test";
		std::string dictDataset = datasetPath + ".dict";

		Vocabulary::Load(vocabulary, dictDataset);
		RandomGenerator::Create(random, vocabulary);

		// Set up the trainer
		FcgmTrainer trainer(&vocabulary, &random);
		trainer.SetNThreads(config.GetInt("threads", 1));
		trainer.SetEpochs(config.GetInt("epochs", 20));
		trainer.SetTau(config.GetInt("tau", 7));
		trainer.SetDelay(config.GetInt("delay", 10));
		trainer.SetEpochSize(config.GetInt("epochSize", 1000000));

		trainer.SetTrainDataset(trainDataset);
		trainer.SetValidDataset(validDataset);
		
		bool incH = config.GetBool("ctxToH"); 
		bool incO = config.GetBool("ctxToO");
		if (!incH && !incO) {
			std::cerr << "Either --ctxToH or --ctxToO should be specified!" << std::endl;
		}

		trainer.SetIncludeH(incH);
		trainer.SetIncludeO(incO);

		// trainer.SetTestDataset(test_dataset);
		
		// Initialize the weights!
		trainer.InitializeWeights((int) vocabulary.vocab_size, config.GetInt("hidden", 100), 
			(int) vocabulary.vocab_size, config.GetString("contextWeights"));
		trainer.SetAlpha(config.GetFloat("alpha", 0.1));
		trainer.SetBeta(config.GetFloat("beta", 0.0000001));

		trainer.Train();
		trainer.WriteModel(config.GetString("output", ""));

	} catch (std::exception e) {
		std::cerr << e.what() << std::endl;
		exit(-1);
	}

	return 0;
}