#pragma once
#include <vector>
#include <vector>
#include <sstream>
#include <fstream>
#include <time.h>
#include "Commons.hpp"
#include "Random.hpp"
#include "ThreadedQueue.hpp"

typedef std::vector<int> Sequence;
typedef std::vector<Sequence> Triple;

class SequenceReader;
class TripleReader;

template<typename T> 
struct Batch {
	int m_size;
	std::vector<T> data;
	Batch() { m_size = 0; }
	template<typename T>
	Batch(Batch<T>&& b) { 
		m_size = b.m_size;
		data = std::move(b.data);
	}

	Batch<T>& operator=(Batch<T>&& b) { 
		this->m_size = b.m_size;
		this->data = std::move(b.data);
		return *this;
	}
};

template<typename T> struct ReaderTraits {};
template<> struct ReaderTraits<SequenceReader> { 
	typedef Batch<Sequence> DataType; 
};
template<>
struct ReaderTraits<TripleReader> {
    typedef Batch<Triple> DataType;
};

template <typename T>
class AbstractReader {
protected:    
	std::default_random_engine rd;
	long long m_read;
	size_t m_mbSize;
	bool m_isReading;
	std::ifstream readerStream;
	ThreadedQueue<T>* m_queue;

public:
	AbstractReader() {
		rd = std::default_random_engine(1);
		m_queue = nullptr;
		m_isReading = false;
		m_mbSize = 1000;
		m_read = 0;
	};

	AbstractReader(ThreadedQueue<T>* queue) : m_queue(queue) {
		rd = std::default_random_engine(1);
		m_isReading = false;
		m_mbSize = 1000;
		m_read = 0;
	};

	void OpenStream(const std::string& fileName) {
		if (m_queue == nullptr) {
			throw std::exception("Cannot open stream without a queue");
		}

		// Initialize Stream
		if (m_isReading) 
			throw std::exception("Consider to close the stream first");

		readerStream = std::ifstream(fileName);
		m_read = 0;
		m_isReading = true;
	}
	
	void SetQueue(ThreadedQueue<T>* q) { m_queue = q; }
	void ResetCount() { m_read = 0; }
	long long GetCount() { return m_read; }
	virtual	bool ReadNextBatch() = 0;

	void SendKillBatch(int nThreads) {
		if (m_queue == nullptr) {
			throw std::exception("Queue was not instantiated");
		}

		for (int i = 0; i < nThreads; i++) {
			T pill;
			m_queue->Push(pill);
		}
	}

	void ResetStream() {
		if (m_isReading) {
			m_read = 0;
			readerStream.seekg(0, readerStream.beg);
		}
	}

	void CloseStream() {
		if (m_isReading) {
			m_read = 0;
			readerStream.close();
			m_isReading = false;
		}
	}
};