#pragma once

#include "Fcgm.hpp"

class DcgmI : public Fcgm {
protected:
	std::vector<int> m_deepDims;
	std::vector<int> m_layerDims;
	std::vector<Matrix*> m_layerWeights;
	std::vector<Matrix*> m_layerBias;
	Activation m_ctxAct;
	
	void DeleteWeights() {
		for (int i = 0; i < m_layerWeights.size(); i++)
			if (m_layerWeights[i] != nullptr)
				delete m_layerWeights[i];
		for (int i = 0; i < m_layerBias.size(); i++)
			if (m_layerBias[i] != nullptr)
				delete m_layerBias[i];
	}

public:
	DcgmI() : Fcgm() {
		m_ctxAct = SIGMOID;
		m_modelName = "DCGM-I";
	}
	DcgmI(int nHid, int nOut) : Fcgm(nHid, nOut) {
		m_ctxAct = SIGMOID;
		m_modelName = "DCGM-I";
	}
	DcgmI(bool proprietaryMemory) : Fcgm(proprietaryMemory) {
		m_ctxAct = SIGMOID;
		m_modelName = "DCGM-I";
	}
	~DcgmI() {
		if (m_proprietaryMemory)
			DeleteWeights();
	}
	
	void SetExternalWeights(const std::vector<int>& layerDims, 
			const std::vector<Matrix*>& layerWeights,
			const std::vector<Matrix*>& layerBias) {
		if (layerDims.size() == 0) {
			throw std::exception("Layer dimensions cannot be 0.");
		}

		m_layerDims = layerDims;
		m_nC = m_layerDims[m_layerDims.size()-1];
		m_layerWeights = layerWeights;
		m_layerBias = layerBias;
	}

	static DcgmI* LoadFromDisk(const std::string& dirPath);

	void SetContextAct(Activation act)
	{
		m_ctxAct = act;
	}
	
protected:
	virtual Layer* BuildContext(const Triple& ctx, NeuralNet& rnn);
};