#pragma once

#include "Fcgm.hpp"

class DcgmII : public Fcgm {
protected:
	std::vector<int> m_deepDims;
	std::vector<int> m_layerDims;
	std::vector<Matrix*> m_layerWeights;
	std::vector<Matrix*> m_layerBias;

	Activation m_ctxAct;

	void DeleteWeights() {
		for (int i = 0; i < m_layerWeights.size(); i++)
			if (m_layerWeights[i] != nullptr)
				delete m_layerWeights[i];
		for (int i = 0; i < m_layerBias.size(); i++)
			if (m_layerBias[i] != nullptr)
				delete m_layerBias[i];
	}

public:
	DcgmII() : Fcgm() {
		m_ctxAct = SIGMOID;
		m_modelName = "DCGM-II";
	}
	DcgmII(int nHid, int nOut) : Fcgm(nHid, nOut) {
		m_ctxAct = SIGMOID;
		m_modelName = "DCGM-II";
	}
	DcgmII(bool proprietaryMemory) : Fcgm(proprietaryMemory) {
		m_ctxAct = SIGMOID;
		m_modelName = "DCGM-II";
	}
	~DcgmII() {
		if (m_proprietaryMemory)
			DeleteWeights();
	}
	
	void SetExternalWeights(const std::vector<int>& layerDims, 
			const std::vector<Matrix*>& layerWeights,
			const std::vector<Matrix*>& layerBias) {
		if (layerDims.size() == 0) {
			throw std::exception("Layer dimensions cannot be 0.");
		}

		m_layerDims = layerDims;
		m_nC = m_layerDims[m_layerDims.size()-1];
		m_layerWeights = layerWeights;
		m_layerBias = layerBias;
	}

	void SetContextAct(Activation fn) {
		m_ctxAct = fn;
	}

public:
	/// public loader for this model
	static DcgmII* LoadFromDisk(const std::string& dirPath);

protected:
	virtual Layer* BuildContext(const Triple& cmr, NeuralNet& network);
};