#pragma once

#include "NnetLib.hpp"
#include "GradientUpdater.hpp"
#include "GradientVisitor.hpp"
#include "Random.hpp"
#include "Rnn.hpp"
#include "AbstractModel.hpp"
#include "AbstractReader.hpp"

class Fcgm : public Rnn {
private:
	Matrix* m_I;

protected:
	Matrix* m_H;
	Matrix* m_O;
	Matrix* m_dO;

	bool m_deepO;
	bool m_includeH;
	bool m_includeO;

	int m_nC;     // units of the context vector

	void DeleteWeights() {
		if (m_I != nullptr) delete m_I;
		if (m_O != nullptr) delete m_O;
		if (m_H != nullptr) delete m_H;
		DeleteRnnWeights();
	}

public:
	Fcgm();
	Fcgm(bool proprietaryMemory);
	Fcgm(int nHid, int nOut);
	Fcgm(const Fcgm& deep_copy) { NOT_IMPLEMENTED; }
	~Fcgm(void) {
		if (m_proprietaryMemory)
			DeleteWeights();
	}

	void SetDeepOutputWeight(Matrix* dO) {
		m_dO = dO;
	}

	void SetContextRnnWeights(Matrix* H, Matrix* O) {
		m_H = H;
		m_O = O;
	}

	void SetIncludeH(bool includeH) {
		m_includeH = includeH;
	}

	void SetDeepO(bool deepO) {
		m_deepO = deepO;
	}

	void SetIncludeO(bool includeO) {
		m_includeO = includeO;
	}

	void SetExternalWeights(Matrix* I) {
		m_I = I;
		m_nC = I->GetRows();
	}

	void SetGenerator(RandomGenerator* gen) {
		m_generator = gen;
	}
	void SetUpdater(IGradientUpdater* upd) {
		m_updater = upd;
	}
	void SetTau(int tau) { m_tau = tau; }

	void CleanNetwork(NeuralNet& rnn, bool cleanError=false);
	virtual void AddStep(const Sequence& s, 
		size_t step, 
		NeuralNet& rnn,
		Layer*& ctxLayer,
		Layer*& lastHiddenLayer,
		bool training=true);

	virtual Layer* BuildContext(const Triple& ctx, NeuralNet& rnn);

	double Train(const Triple& seq);
	double Test(const Triple& seq, bool stoch=false);
	
	virtual std::vector<Candidate> GenerateNextWord(const Triple& ctx, PastInformation* info, int currentInput, int nK=100, bool sample=false);
	static Fcgm* LoadFromDisk(const std::string& dirPath);
};