#pragma once

#include "DcgmTrainer.hpp"
#include "print.hpp"
#include "Rcgm.hpp"
#include <chrono>

class RcgmTrainer : public FcgmTrainer {
protected:
	int m_nsHid;

	Matrix WShh;
	Matrix WSin;
	Matrix bshh;

public:
	RcgmTrainer(Vocabulary* v, RandomGenerator* g) : FcgmTrainer(v, g) {}

	void InitializeWeights(int hidden) { 
		m_nHid = hidden;
		m_nIn = m_vocabulary->vocab_size;
		m_nOut = m_nIn;
		m_nsHid = m_nHid;

		InitializeRnnWeights(m_nIn, m_nHid, m_nOut);
		
		std::cerr << "Initializing model RCGM-" << m_nIn << ":" << m_nHid << ":" << m_nOut << ":" << m_nsHid;
		
		WSin.Resize(m_nsHid, m_nHid);
		WSin.SetName("S_in");

		WShh.Resize(m_nsHid, m_nsHid);
		WShh.SetName("S_hh");
		
		bshh.Resize(m_nsHid, 1);
		bshh.SetName("bs_hh");

		random_fill_policy(WShh.GetDataForUpdate(), WShh.GetSize());
		random_fill_policy(WSin.GetDataForUpdate(), WSin.GetSize());
		orthogonal_projection(WShh.GetRows(), WShh.GetCols(), WShh.GetDataForUpdate());

		H.Resize(m_nHid, m_nsHid);
		H.SetName("H");
		random_fill_policy(H.GetDataForUpdate(), H.GetSize());
	}

	void InitializeWeights(std::string& pretrainedRnn) { 
		Whh.LoadFromExternalBinary(pretrainedRnn + "\\w_hh.bin");//Whh.InPlaceTranspose();
		Win.LoadFromExternalBinary(pretrainedRnn + "\\w_in.bin");
		Wout.LoadFromExternalBinary(pretrainedRnn + "\\w_out.bin");
		
		bhh.LoadFromExternalBinary(pretrainedRnn + "\\b_hh.bin");
		bout.LoadFromExternalBinary(pretrainedRnn + "\\b_out.bin");
		
		m_nHid = Whh.GetRows();
		m_nIn = Wout.GetRows();
		m_nOut = m_nIn;
		m_nsHid = m_nHid;

		std::cerr << "Initializing model RCGM-" << m_nIn << ":" << m_nHid << ":" << m_nOut << ":" << m_nsHid;
				
		WSin.Resize(m_nsHid, m_nHid);
		WSin.SetName("S_in");

		WShh.Resize(m_nsHid, m_nsHid);
		WShh.SetName("S_hh");
		
		bshh.Resize(m_nsHid, 1);
		bshh.SetName("bs_hh");

		random_fill_policy(WShh.GetDataForUpdate(), WShh.GetSize());
		random_fill_policy(WSin.GetDataForUpdate(), WSin.GetSize());
		orthogonal_projection(WShh.GetRows(), WShh.GetCols(), WShh.GetDataForUpdate());

		H.Resize(m_nHid, m_nsHid);
		H.SetName("H");
		random_fill_policy(H.GetDataForUpdate(), H.GetSize());
	}

	void DoJob(AbstractTrainer<TripleReader>& parentCaller, 
		ThreadedQueue<Batch<Triple>>& workingQueue,
		bool isTraining=false);

	void WriteModel(const std::string& outputPath);
	void PrintStatus(bool training=true);
	
	~RcgmTrainer() {}
};