#pragma once

#include "Vocabulary.hpp"
#include "Random.hpp"
#include "NnetLib.hpp"
#include "GradientVisitor.hpp"
#include "GradientUpdater.hpp"
#include "AbstractReader.hpp"
#include "AbstractModel.hpp"
#include <map>

class Rnn : public AbstractModel {
protected:
	Matrix* m_Win;
	Matrix* m_Wout;
	Matrix* m_Whh;
	Matrix* m_Bout;
	Matrix* m_Bhh;

	Activation m_act;

	std::vector<NCELossLayer*> m_nceLayers;
	std::vector<CrossEntropyLossLayer*> m_ceLayers;

	int m_nIn;
	int m_nOut;
	int m_nHid;
	int m_tau;

	void DeleteRnnWeights() {
		if (m_Bhh != nullptr) delete m_Bhh;
		if (m_Bout != nullptr) delete m_Bout;
		if (m_Whh != nullptr) delete m_Whh;
		if (m_Wout != nullptr) delete m_Wout;
		if (m_Win != nullptr) delete m_Win;
	}

	void CleanNetwork(NeuralNet& rnn, bool cleanError=false);

private:
	void AddStep(const Sequence& sequence, size_t step, 
		Layer*& lastHiddenLayer, NeuralNet& rnn, bool training=true);

public:
	Rnn();
	Rnn(bool proprietaryMemory);
	Rnn(int nHid, int nOut);

	~Rnn(void) {
		if (m_proprietaryMemory) 
			DeleteRnnWeights();
	}

	void SetTiled(Matrix* w) {
		// tiled updates for hidden weights
		v.__create_tiled(w, (m_tau + 1) * 100);
	}

	void SetTau(int tau) { m_tau = tau; }
	void SetExternalRnnWeights(Matrix* in, Matrix* hh, Matrix* out) {		
		m_Win = in;
		m_Wout = out;
		m_Whh = hh;

		m_nIn = in->GetCols();
		m_nOut = out->GetRows();
		m_nHid = hh->GetRows();
	};

	void SetExternalBias(Matrix* bhh, Matrix* bout) {
		m_Bhh = bhh;
		m_Bout = bout;
	};

	virtual double Test(const Sequence& seq, bool stoch=false);
	virtual double Test(const Triple& tri, bool stoch=false);
	double Train(const Sequence& seq);

	virtual std::vector<Candidate> 
	Rnn::GenerateNextWord(const Triple& ctx, PastInformation* info, int currentInput, size_t nK=100);
	
	static Rnn* LoadFromDisk(const std::string& dirPath);

	void SetAct(Activation fn) {
		m_act = fn;
	}
	std::string GetAct() {
		return m_act;
	}
};