
#pragma once

#include "AbstractTrainer.hpp"
#include "GradientUpdater.hpp"
#include "SequenceReader.hpp"
#include "Rnn.hpp"
#include <chrono>

class RnnTrainer : public AbstractTrainer<SequenceReader> {
protected:
	int m_tau;
	int m_nIn; 
	int m_nHid; 
	int m_nOut;

	Activation m_act;

	Matrix Win;
	Matrix Wout;
	Matrix Whh;

	Matrix bhh;
	Matrix bout;

public:
	RnnTrainer(Vocabulary* v, RandomGenerator* g) : AbstractTrainer(v, g) {
		m_act = SIGMOID;
		m_tau = 7;
	}

	void InitializeWeights(int nIn, int nHid, int nOut) { 
		m_nIn = nIn;
		m_nHid = nHid;
		m_nOut = nOut;
		
		Win.Resize(m_nHid, m_nIn);
		Win.SetName("W_in");
		
		Wout.Resize(m_nOut, m_nHid);
		Wout.SetName("W_out");
		
		Whh.Resize(m_nHid, m_nHid);
		Whh.SetName("W_hh");
		
		bhh.Resize(m_nHid, 1);
		bhh.SetName("b_hh");
		
		bout.Resize(m_nOut, 1);
		bout.SetName("b_out");
		
		if (m_act == SIGMOID) {
			random_fill_policy(Win.GetDataForUpdate(), Win.GetSize());
			random_fill_policy(Wout.GetDataForUpdate(), Wout.GetSize());
			random_fill_policy(Whh.GetDataForUpdate(), Whh.GetSize());
			orthogonal_projection(Whh.GetRows(), Whh.GetCols(), Whh.GetDataForUpdate());
		} else {
			random_fill_policy(Win.GetDataForUpdate(), Win.GetSize());
			random_fill_policy(Wout.GetDataForUpdate(), Wout.GetSize());
			random_fill_policy(Whh.GetDataForUpdate(), Whh.GetSize());
		}
	}

	void SetActivation(const std::string& act) 
	{ 
		if (act == "sigmoid") m_act = SIGMOID; 
		else if (act == "tanh") m_act = TANH;
		else if (act == "relu") m_act = ReLU;
	}

	void SetTau(int tau) { m_tau = tau; }
	void DoJob(AbstractTrainer<SequenceReader>& parentCaller, 
		ThreadedQueue<Batch<Sequence>>& workingQueue,
		bool isTraining=false);
	void WriteModel(const std::string& outputPath);

	void PrintStatus(bool training=true);

	~RnnTrainer() {}
};