#include "TGTReader.hpp"

#include <sstream>

bool TGTReader::ReadNextCandidate(HeadRecord& x) {
	if (m_startRead == false) {
		m_fileH = std::ifstream(m_fileName);
		if (!m_fileH.good()) {
			std::cerr << m_fileName << std::endl;
			std::cerr << "The specified nbest file was not found!" << std::endl;
			exit(-1);	
		}

		m_startRead = true;
	}

	bool referenceFound = false;
	bool eof = false;
	std::string line = "";
	
	x = HeadRecord();
	while (std::getline(m_fileH, line)) {
		if (line == "" && x.rlist.size() > 0) {
			referenceFound = true;
			break;
		}

		// This should be the source
		if (x.mtext == "") {
			x.mtext = line;
			continue;
		}
		
		size_t pos = 0;
		if ((pos=line.find("\\CTX")) != std::string::npos) {
			x.ctext = line.substr(pos+5);
			continue;
		}
		
		if ((pos=line.find("\\REF")) != std::string::npos) {
			x.rtext.push_back(line.substr(pos+5));
			continue;
		}

		if ((pos=line.find("@@H")) != std::string::npos) {
			line = line.substr(pos+3);
			std::stringstream ss(line);
			std::string featureName;
			while (std::getline(ss, featureName, ',')) {
				x.featureNames.push_back(featureName);
			}
			continue;
		}

		if (line.find("@@Q") != std::string::npos) {
			// Start parsing tail records
			while (1)
			{
				std::getline(m_fileH, line);

				// We reached the end
				if (line.find("@@===") != std::string::npos)
					break;

				TailRecord t; t.rtext = line;
				
				std::getline(m_fileH, line);
				pos = 0;
				
				if ((pos = line.find("@@V")) != std::string::npos) {
					line = line.substr(pos+3);
					std::stringstream ss(line);
					std::string featureValue;
					while (std::getline(ss, featureValue, ',')) {
						t.featureValues.push_back(std::atof(featureValue.c_str()));
					}
				} else {
					std::cerr << "Unexpected line " << line << std::endl;
					throw std::runtime_error("TGT format error!");
				}

				x.rlist.push_back(t);
			}
			continue;
		}

		if (line.find("-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-") != std::string::npos) {
			continue;		
		}
	}

	if (x.ctext == "" && referenceFound)
	{
		throw std::exception("Context not found in source tgt, are you sure that a \\CTX tag is present?");
	}

	return referenceFound;
}