#pragma once

#include "AbstractWriter.hpp"

class TGTWriter : public AbstractWriter {
private:
	bool m_startWriting;

public:
	TGTWriter(std::string outputFile) : AbstractWriter(outputFile) {
		m_startWriting = true;
	}
		
	void WriteRecord(const HeadRecord& x) {
		m_outputWriter << x.mtext << std::endl;	
		for (int i = 0; i < x.rtext.size(); i++)
			m_outputWriter << "\\REF " << x.rtext[i] << std::endl;
		m_outputWriter << "\\CTX " << x.ctext << std::endl;
		
		m_outputWriter << "@@H";
		int elt = (int) x.featureNames.size();
		for (auto it = x.featureNames.begin(); it != x.featureNames.end(); it++) {
			m_outputWriter << *it;
			if (--elt != 0) m_outputWriter << ",";
		}
		
		m_outputWriter << std::endl;

		m_outputWriter << "@@Q" << std::endl;
		for (int i = 0; i < x.rlist.size(); i++) 
		{
			m_outputWriter << x.rlist[i].rtext << std::endl;
			
			m_outputWriter << "@@V";
			size_t elt = x.rlist[i].featureValues.size();
			for (auto it = x.rlist[i].featureValues.begin(); 
				it != x.rlist[i].featureValues.end(); it++) 
			{
				m_outputWriter << *it;
				if (--elt != 0) m_outputWriter << ",";
			}
			
			m_outputWriter << std::endl;
		}

		m_outputWriter << "@@===" << std::endl;
		m_outputWriter << "  -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-" << std::endl;
		if (x.best != "") m_outputWriter << x.best << std::endl;
		m_outputWriter << "generate2 {}" << std::endl;
		m_outputWriter << "====================================================" << std::endl;
		m_outputWriter << std::endl;
	}

	~TGTWriter() { m_outputWriter.close(); }
};
