/// 
/// 
/// 

#pragma once

#include <queue>
#include <algorithm>
#include <mutex>
#include <set>

template <class DataType, 
class Container=std::vector<DataType>,
class DataCompare=std::less<Container::value_type>>
class ThreadedFixedPQ {
public:
	void Push(DataType& value) {
		m_lock.lock();
		m_queue.push(value);
		if (m_queue.size() > m_maxSize)
			m_queue.pop();
		m_lock.unlock();
	}

	std::vector<DataType> PopAll() {
		m_lock.lock();
		size_t l = m_queue.size();
		std::vector<DataType> v(l);
		while (m_queue.size() != 0)
		{
			v[--l] = m_queue.top();
			m_queue.pop();
		}

		m_lock.unlock();
		return v;
	}

	size_t GetSize() {
		return m_queue.size();
	}

	void SetSize(size_t maxSize) {
		m_maxSize = maxSize;
	}

	bool IsEmpty()
	{
		return m_queue.empty();
	}

	ThreadedFixedPQ(size_t maxSize) : m_maxSize(maxSize) {}
	ThreadedFixedPQ() : m_maxSize(100) {}
	ThreadedFixedPQ(ThreadedFixedPQ&& shallowCopy)
	{
		this->m_maxSize = shallowCopy->maxSize;
		this->m_queue = shallowCopy->m_queue;
	}

protected:
	struct ReversedCompare : DataCompare {
		bool operator() (const DataType& l, const DataType& r)
		{
			return DataCompare::operator()(r, l);
		}  

		bool operator() (const DataType& l, const DataType& r) const {
			return DataCompare::operator()(r, l);
		}

		bool operator() (DataType& l, DataType& r) {
			return DataCompare::operator()(r, l);
		}

		bool operator() (DataType& l, DataType& r) const {
			return DataCompare::operator()(r, l);
		}
	};

	std::priority_queue<DataType, Container, ReversedCompare> m_queue;
	std::mutex m_lock;
	size_t m_maxSize;
};