#pragma once

#include <mutex>
#include <condition_variable>
#include <list>

template <typename T> 
class ThreadedQueue {
private:
	std::list<T> m_queue;
	std::mutex m_mtx;
	size_t m_maxSize;
	int m_nWaiting;

	std::condition_variable m_empty;
	std::condition_variable m_full;

public:
	ThreadedQueue() { m_maxSize = 10; m_nWaiting = 0; }
	ThreadedQueue(size_t max) : m_maxSize(max) {}
	~ThreadedQueue() {}

	size_t GetElementSize() { return m_queue.size(); }

	void Push(const T& item) {
		std::unique_lock<std::mutex> lck(m_mtx);
		while (m_queue.size() >= m_maxSize)
			m_empty.wait(lck);

		m_queue.push_back(item);
		lck.unlock();
		m_full.notify_one();
	}

	bool IsEmpty() {
		return m_queue.size() == 0;
	}
	
	int GetWaiting() {
		return m_nWaiting;
	}

	T PopFront() {
		std::unique_lock<std::mutex> lck(m_mtx);
		m_nWaiting += 1;
		while (m_queue.size() <= 0)
			m_full.wait(lck);
		m_nWaiting -= 1;

		T item = static_cast<T&&>(m_queue.front());
		m_queue.pop_front();
		lck.unlock();
		m_empty.notify_one();
		return item;
	}
};