//
//  print.h
//  Rnn
//
//  Created by Alessandro Sordoni on 3/11/2014.
//  Copyright (c) 2014 Alessandro Sordoni. All rights reserved.
//

#pragma once 

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

/*
#include "Vocabulary.hpp"
#include "DenseMatrix.hpp"
#include <iostream>

template<typename T>
void print_vector(T* v, int len) {
    for (int i = 0; i < len; i++) {
        std::cout << v[i] << " ";
    }
    std::cout << std::endl;
}

void serialize_matrix(const int isize,
               const int osize,
               double * param,
               std::string output_path) {

	FILE *fo;
    fopen_s(&fo, output_path.c_str(), "wb");
    if (fo == NULL) {
        printf("Error opening file %s\n", output_path.c_str());
        printf("Quit");
        return;
    }
    
    fprintf(fo, "%d %d\n", isize, osize);
    for (int i = 0; i<isize*osize; i++) {
        const double value = (double) *(param+i);
        fwrite(&value, sizeof(double), 1, fo);
    }
    
    fclose(fo);
}

void serialize_embeddings(const int isize,
                      const int osize,
                      double * param,
                      std::vector<vocab_word>& vocab,
                      std::string output_path) {
    FILE *fo;
    fopen_s(&fo, output_path.c_str(), "wb");
    if (fo == NULL) {
        printf("Error opening file %s\n", output_path.c_str());
        printf("Quit");
        return;
    }
    
    fprintf(fo, "%d %d\n", (int)isize, (int)osize);
    for (int i = 0; i<isize; i++) {
        fprintf(fo, "%s ", vocab[i].word.c_str());
        for (int z = 0; z<osize; z++) {
            // Convert to double here!
            const double value = (double) *(param+osize*i+z);
            fwrite(&value, sizeof(double), 1, fo);
        }
        
        fprintf(fo, "\n");
    }
    
    fclose(fo);
}

void serialize_model(const int n_in,
                     const int n_hid,
                     const int n_out,
                     const double * W_in,
                     const double * W_out,
                     const double * W_hh,
                     std::vector<vocab_word>& vocab,
                     std::string output_path) {
    FILE *fo;
    fopen_s(&fo, output_path.c_str(), "wb");
    if (fo == NULL) {
        std::string error_msg = "Error opening file: " + output_path;
        throw std::exception(error_msg.c_str());
    }
    
    fprintf(fo, "%d %d %d\n", (int)n_in, (int)n_hid, (int)n_out);
    
    // Write W_in matrix
    for (int i = 0; i<n_in; i++) {
        fprintf(fo, "%s ", vocab[i].word.c_str());
        for (int z = 0; z<n_hid; z++) {
            const double value = (double) *(W_in + n_hid*i + z);
            fwrite(&value, sizeof(double), 1, fo);
        }
        
        fprintf(fo, "\n");
    }
    
    // Write W_hh matrix
    for (int i = 0; i<n_hid; i++) {
        for (int z = 0; z<n_hid; z++) {
            const double value = (double) *(W_hh + n_hid*i + z);
            fwrite(&value, sizeof(double), 1, fo);
        }
        
        fprintf(fo, "\n");
    }
    
    // Write W_out matrix
    for (int i = 0; i<n_out; i++) {
        for (int z = 0; z<n_hid; z++) {
            const double value = (double) *(W_out + n_hid*i + z);
            fwrite(&value, sizeof(double), 1, fo);
        }
        
        fprintf(fo, "\n");
    }
    
    fclose(fo);
}

void load_matrix(std::string input_path, DenseMatrix*& M) {
    int num = 0, m_size = 0;
    FILE *f;
    fopen_s(&f, input_path.c_str(), "rb");
    if (f == NULL) {
        printf("Input file not found\n");
        return;
    }
    
    char ch;
    fscanf(f, "%lld", &num);
    fscanf(f, "%lld", &m_size);
    fscanf(f, "%c", &ch);
    
    // TODO: Put name in the file
    M = new DenseMatrix(num, m_size, "");
    for (int b = 0; b < num; b++) {
        for (int a = 0; a < m_size; a++) {
            double n = 0;
            fread(&n, sizeof(double), 1, f);
			M->GetElementForUpdate(b, a) = n;
        }
    }
    fclose(f);
}*/

inline int IsInteger(const std::string& s) {
	return !s.empty() && std::find_if(s.begin(), 
        s.end(), [](char c) { return !isdigit(c); }) == s.end();
}

inline int LoadFromExternalEmbeddings(std::string inputPath, Vocabulary& v, Matrix* Emb) {
    int num = 0, size = 0, founds = 0;

    std::ifstream input(inputPath, std::ios_base::binary);
	input>>num>>size;

	Emb->Resize(num, size);
	double* rawData = Emb->GetDataForUpdate();

	for (int b = 0; b < num; b++) {
		std::string token;
 		
		input>>token;

		std::vector<double> extEmb(size);
		for (int a = 0; a < size; a++) {
			double n = 0;
			input>>n;
			extEmb[a] = n;
		}
        
		double len = 0;
        for (int a = 0; a < size; a++) len += extEmb[a] * extEmb[a];
        len = sqrt(len);
        for (int a = 0; a < size; a++) extEmb[a] /= len;
		
		// Copy into Emb
		if (v.inv_vocab.find(token) != v.inv_vocab.end()) {
			memcpy(&rawData[size*v.inv_vocab[token]], &extEmb[0], size*sizeof(double));
			founds+=1;
		}
    }
    
	input.close();
	return founds;
}
