#ifndef nnet_alg_hpp
#define nnet_alg_hpp

#define _USE_MATH_DEFINES
#include <cmath>
#include <stdlib.h>
#include <cstring>
#include <iostream>

#ifdef CBLAS
//#include <acml.h>
#ifdef MKL
#include <mkl.h>
#else
#include <cblas.h>
#include <complex>
#define lapack_complex_float std::complex<float>
#define lapack_complex_double std::complex<double>
#include <lapacke.h>
#endif

// Compatibility
//#define CblasNoTrans 'N'
//#define CblasTrans 'T'
//#define CblasConjTrans = 'C'
//#define CblasRowMajor 101
//#define CblasColMajor 102
//#define CblasUpper 'U'
#endif


//*********************
// FILL POLICIES
double * create_tensor(int rows, int cols, void (*fill_policy)(double *, int));
void no_fill_policy(double* tensor, int m_size);
void max_min_policy(double* tensor, int m_size, double min, double max);
void one_fill_policy(double* tensor, int m_size);
void zero_fill_policy(double* tensor, int m_size);
void random_fill_policy(double * tensor, int m_size, float mean=0.f, float sigma=0.1f);
//*********************

//*********************
// ALG
// fast exp() implementation
// from rnnmlib
static union{
    double d;
    struct{
        int j,i;
    } n;
} d2i;
#define EXP_A (1048576/M_LN2)
#define EXP_C 60801
#define FAST_EXP(y) (d2i.n.i = EXP_A*(y)+(1072693248-EXP_C),d2i.d)
#define MAX(a,b) ((a) > (b) ? (a) : (b))
#define MIN(a,b) ((a) < (b) ? (a) : (b))
//

double vvdot(int N,  double *a,  double *b);
void softmax(int N,  double *a, double *out);
void tacc(int N,  double f,  double *a, double *b);
void sigmoid(int N,  double *a, double *output);
void gendot(int s1, int m, int s2,  double *W,  double *U, double *output, bool Wt=false, bool Ut=false);
void sprod(int N,  double *a, double s, double *output);
void truncate(int N,  double *a, double min, double max, double *output);
void ArgMax(int N,  double *v, int& max);
void onehot(int N, int elt, double* o);
void mvdot(int s1, int s2,  double* M,  double* x, double* output, bool Wt=false);
void logadd(double& a, double b);

/***************** WRAPPERS ****************/
void __sigmoid__(int N,  double* elt, double *output);
void __tanh__(int N,  double* elt, double *output);
void __onehot__(int N, int elt, double *o);
void __dadd__(int N,  double *a,  double s, double *output);
void __cacc__(int N, int col_id, double *a, double f, double *x);
void __dcopy__(int N,  double *a, double *output);
void __dscal__(int N,  double *a,  double s, double *output);
void __tacc__(int N,  double f,  double *a, double *b);
void __mt__( double* A,  int r,  int c, double* B);
void __gendot__(int s1, int m, int s2,  double *W,  double *U, double *output,
                bool Wt=false, bool Ut=false);
void __mvdot__(int s1, int s2,  double *M,  double *x, double *output,
               bool Mt=false);
void __sparsemult__( int* idx,  double* values, int len, int rows, int cols,  double *M, double *output);
double __norm2__(int N,  double *X);
void __dset__(int N, int alpha, double* tensor);
void __had__(int N,  double *x,  double *y, double *output);
void __vvoutdot__(int s1, int s2,  double *x,  double *y, double *output);
void __argmax__(int N,  double* v, int& max);
void __softmax__(int N,  double *a, double *out);
void __truncate__(int N,  double *a, double min, double max, double *output);
double __vvdot__(int N,  double *a,  double *b);
double __d__(int N,  double *a,  double *b);
void __svd__(int m, int n, double *M, double*& U, double*& s, double*& Vt, int &num_s);

void orthogonal_projection( int r, int c, double *W );
void normalize_rows( int r, int c, double *W );
void normalize_cols( int r, int c, double *W );
/***********************************************/
#endif
