
#include "DenseMatrix.hpp"

DenseMatrix::DenseMatrix(int rows, int cols) {
	m_name = "@@@";
	m_numRows = 0; 
	m_numCols = 0; 
	m_size = 0;	
	m_data = nullptr;
	Resize(rows, cols);
}

DenseMatrix::DenseMatrix(int rows, int cols, std::string name) {
	m_name = name;
	m_data = nullptr;
	Resize(rows, cols);
}

DenseMatrix::DenseMatrix() {
	m_numRows = 0; 
	m_numCols = 0; 
	m_size = 0;
	m_name = "@@@";
	m_data = nullptr;
};

DenseMatrix::DenseMatrix(const DenseMatrix& deep_copy) {
	m_numRows = 0; 
	m_numCols = 0; 
	m_size = 0;	
	m_data = nullptr;
	Resize(deep_copy.m_numRows, deep_copy.m_numCols);
	m_name = deep_copy.m_name;
	SetData(deep_copy.m_data);
};

DenseMatrix::~DenseMatrix() { 
	if (m_data != nullptr) delete m_data;
	m_data = nullptr;
}

void DenseMatrix::Resize(int rows, int cols) { 
	if (m_size < rows*cols || m_data == nullptr) {
		if (m_data != nullptr) {
			delete m_data;
		}
		
		m_data = (double*) malloc (rows*cols*sizeof(double));
		__dset__(rows*cols, 0, m_data);
	}

	m_numRows = rows; 
	m_numCols = cols;
	m_size = rows*cols;
}
void DenseMatrix::ResetValues() 
{
	__dset__(m_size, 0, &m_data[0]);
}
/** Getters and setters **/
double& DenseMatrix::GetElementForUpdate(int i, int j) {
	if (i >= m_numRows || j >= m_numCols) 
		throw std::exception("Element out of range");
	return m_data[i*m_numCols + j];
}
double DenseMatrix::GetElement(int i, int j) const {
	if (i >= m_numRows || j >= m_numCols) 
		throw std::exception("Element out of range");
	return m_data[i*m_numCols + j];
}
double* DenseMatrix::GetRow(int i) {
	if (i >= m_numRows) { 
		std::cerr<<GetName()<<std::endl;
		std::cerr<<m_numCols<<std::endl;
		throw std::exception(("Row out of range " + std::to_string(m_numRows) + 
		   " requested " + std::to_string(i)).c_str());
	}
	
	return &m_data[i*m_numCols];
}
void DenseMatrix::SetData(double* data) { 
	if (m_data == nullptr)
		throw std::exception("Matrix should be initialized first!");
	__dcopy__(m_size, data, m_data); 
}
double* DenseMatrix::GetDataForUpdate() { 
	return m_data; 
}
std::string DenseMatrix::ToString() {
	std::stringstream ss;
	ss << "DenseMatrix(" << GetRows() << "," << GetCols() << ") - " << GetName() << " - " << "\n";
	for (int r = 0; r < GetRows(); r++) {
		for (int c = 0; c < GetCols(); c++) {
			ss << GetElement(r, c) << " ";
		}
		if (r != GetRows() - 1) ss << "\n";
	}
	ss << "\n";
	return ss.str();
}

bool DenseMatrix::LoadFromExternalBinary(const std::string& path) {
	std::ifstream input(path, std::ios::binary);
	if (!input.good()) return false;
	input >> *this;
	input.close();
	return true;
}

void DenseMatrix::WriteToExternalBinary(const std::string& path) {
	std::ofstream output(path, std::ios::binary);
	output << *this;
	output.close();
}

/** ewise operations **/
void DenseMatrix::InPlaceScale(double f) {
	__dscal__(m_size, GetDataForUpdate(), f, GetDataForUpdate());
}
void DenseMatrix::InPlaceTranspose() {
	//double* temp = (double*)malloc(m_size*sizeof(double));
	//memcpy(temp, m_data, m_size*sizeof(double));
	//__mt__(temp, m_numRows, m_numCols, m_data);
	// only swap the columns
	m_numRows = m_numRows+m_numCols;
	m_numCols = m_numRows-m_numCols;
	m_numRows = m_numRows-m_numCols;
	// delete temp;
}
void DenseMatrix::InPlaceAdd(double f) {
	for (int i = 0; i < m_size; i++)
		m_data[i] += f;
}
void DenseMatrix::InPlacePow(double n) {
	for (int i = 0; i < m_size; i++)
		m_data[i] = pow(m_data[i], n);
}
double DenseMatrix::Norm2() {
	return __norm2__(m_size, &m_data[0]);
}
void DenseMatrix::InPlaceClip(double min, double max) {
	for (int i = 0; i < m_size; i++) {
		if (m_data[i] < min) m_data[i] = min;
		else if (m_data[i] > max) m_data[i] = max;
	}
}
void DenseMatrix::InPlaceClip(double threshold) {
	double n = Norm2();
	if (n > threshold)
		InPlaceScale(threshold/n);
}
int DenseMatrix::ArgMax() {
	int i = 0;
	__argmax__(m_size, m_data, i);
	return i;
}
