#pragma once

#include "Commons.hpp"
#include "Alg.hpp"
#include <unordered_map>
#include <sstream>
#include <fstream>

// Just a general dense matrix.
class DenseMatrix {
	double* m_data;          /* the actual data */
	int m_numRows;
	int m_numCols;
	int m_size;
	std::string m_name;

public:
	DenseMatrix(int rows, int cols);
	DenseMatrix(int rows, int cols, std::string name);
	DenseMatrix();
	DenseMatrix(const DenseMatrix& deep_copy);
	~DenseMatrix();

	void Resize(int rows, int cols);
	void ResetValues();

	/** Getters and setters **/
	double& GetElementForUpdate(int i, int j);
	double GetElement(int i, int j) const;
	void SetData(double* data);
	double* GetDataForUpdate();
	double* GetRow(int i);
	/*************************/
	
	int GetRows() { return m_numRows; }
	int GetSize() { return m_size; }
	int GetCols() { return m_numCols; }
	std::string GetName() { return m_name; }
	void SetName(const std::string& name) { m_name = name; }

	std::string ToString();
	friend std::ostream& operator<<(std::ostream& stream, const DenseMatrix& us)
	{
		stream<<us.m_name<<" "<<us.m_numRows<<" "<<us.m_numCols<<std::endl;
		for (int i=0;i<us.m_numRows;++i)
		{
			for (int j=0;j<us.m_numCols;++j) 
			{
				float v = (float) us.GetElement(i,j);
				stream.write(reinterpret_cast<char*>(&v), sizeof(float));
			}
		}
		stream<<std::endl;
		return stream;
	}

	friend std::istream& operator>>(std::istream& stream, DenseMatrix& us)
	{
		stream>>us.m_name>>us.m_numRows>>us.m_numCols;
		stream.get();

		us.Resize(us.m_numRows, us.m_numCols);
		double * data = us.GetDataForUpdate();

		for (int i=0;i<us.m_size;++i)
		{
			float value = 0;
			stream.read(reinterpret_cast<char *>(&value), sizeof(float));
			data[i] = value;	
		}

		return stream;
	}

	// External compatibility operation
	bool LoadFromExternalBinary(const std::string& path);
	void WriteToExternalBinary(const std::string& path);
	void LoadFromExternalEmbeddings(const std::string& path);

	/** ewise operations **/
	void CopyFrom(double* e);
	void InPlaceTranspose();
	void InPlaceScale(double f);
	void InPlaceAdd(double f);
	void InPlacePow(double n);
	double Norm2();
	void InPlaceClip(double threshold);
	void InPlaceClip(double min, double max);
	int ArgMax();
};