//
//  rnn_entropy.cpp
//  Rnn
//
//  Created by Alessandro Sordoni on 3/10/2014.
//  Copyright (c) 2014 Alessandro Sordoni. All rights reserved.
//

#include <iostream>
#include <vector>
#include <unordered_map>
#include <string>
#include <algorithm>
#include <iomanip>
#include <iterator>
#include <thread>
#include <sstream>
#include <thread>
#include <mutex>

#include "RnnTrainer.hpp"

int debug = 0;
int n_in;                  // n_in units (n of words!)
int n_out = 50;             // n_out units
int n_hid = 100;           // n_hid units
double alpha = 1e-2;         // gradient step
double beta = 1e-6;
int tau = 5;
int reconstruct_rate = 100000;

double error = 0.;
double predictions = 0.;
long long words_seen = 0;

// Work queue
Vocabulary vocabulary;
RandomGenerator random;

int main(int argc, char* argv[])
{
	int threads = 1;
	std::string id = "unknown";
	int mb = 100;
	int epochs = 10;
	std::string output_path = "";
	std::string dataset_path = "";

	if (argc == 1) {
		// Print help here!
		exit(1);
	}

	Config config = Config::ReadCommandLine(argc, argv);

	std::string datasetPath = config.GetString("train");

	std::string train_dataset = datasetPath + ".train";
	std::string valid_dataset = datasetPath + ".valid";
	std::string test_dataset = datasetPath + ".test";
	std::string dict_dataset = datasetPath + ".dict";
	
	if (debug > 1) std::cout << "Loading vocabulary " << std::endl;

	Vocabulary::Load(vocabulary, dict_dataset);
	RandomGenerator::Create(random, vocabulary);
	
	// Set up the trainer
	RnnTrainer trainer(&vocabulary, &random);
	trainer.SetNThreads(config.GetInt("threads", 1));
	trainer.SetEpochs(config.GetInt("epochs", 10));
	trainer.SetEpochSize(config.GetInt("epochSize", 1000000));
	trainer.SetTau(config.GetInt("tau", 7));
	trainer.SetDelay(config.GetInt("delay", 1000));
	trainer.SetActivation(config.GetString("activation", "sigmoid"));
	
	trainer.SetTrainDataset(train_dataset);
	trainer.SetValidDataset(valid_dataset);
	trainer.SetTestDataset(test_dataset);

	// Initialize the weights!
	trainer.InitializeWeights((int) vocabulary.vocab_size, config.GetInt("hidden", 100), (int) vocabulary.vocab_size);
	trainer.SetAlpha(config.GetFloat("alpha", 0.1));
	trainer.SetBeta(config.GetFloat("beta", 0.0000001));
	
	trainer.Train();
	// trainer.Test();
	trainer.WriteModel(config.GetString("output", ""));

	return 0;
}