#pragma once

class AbstractModel {
protected:
	std::string m_modelName;
	std::unordered_map<std::string, CudaMatrix*> m_weights;

	RandomGenerator* m_generator;
	Vocabulary* m_vocabulary;
	IGradientUpdater* m_updater;
	GradientVisitor v;
	bool m_proprietaryMemory;

public:
	AbstractModel() {
		m_proprietaryMemory = false;
		m_updater = nullptr;
		m_vocabulary = nullptr;
		m_generator = nullptr;
		m_modelName = "";
	};

	AbstractModel(bool proprietaryMemory) {
		m_proprietaryMemory = proprietaryMemory;
		m_updater = nullptr;
		m_vocabulary = nullptr;
		m_generator = nullptr;
		m_modelName = "";
	};

	~AbstractModel() {
		if (m_proprietaryMemory)
		{
			if (m_vocabulary) delete m_vocabulary;
			if (m_generator) delete m_generator;
			if (m_updater) delete m_updater;
		}
	}

	std::string GetModelName() { return m_modelName; }
	void SetVocabulary(Vocabulary* o) { m_vocabulary = o; }
	Vocabulary* GetVocabulary() { return m_vocabulary; }
	void SetUpdater(IGradientUpdater* upd) { m_updater = upd; }
	void SetGenerator(RandomGenerator* g) { m_generator = g; }
	
	virtual float Evaluate(const Batch<Triple>& batch, bool stochastic) { return 0.f; };
	virtual float Run(const Batch<Triple>& data, bool isTraining=false) { return 0.f; };
	virtual float Run(const Batch<Sequence>& data, bool isTraining=false) { return 0.f; };
};