#include "DcgmI.hpp"

Layer* DcgmI::ComputeContext() {
	// Build the context vector
	SparseInputLayer* cmInput = m_trainRnn->AddLayer(new SparseInputLayer("cm_i"));

	std::vector<BiasLayer*> biases;

	for (int i = 0; i < m_layerWeights.size(); i++)
	{
		std::string id = std::to_string(i);
		BiasLayer* b = m_trainRnn->AddLayer(new BiasLayer(m_batchSize, "b_" + id));
		m_biasLayers.push_back(b);
		biases.push_back(b);
	}

	Layer* lastLayer = cmInput;
	for (size_t i = 0; i < m_layerWeights.size(); i++) {
		std::string id = std::to_string(i);
		
		HiddenLayer* h = m_trainRnn->AddLayer(new HiddenLayer((i == 0) ? ID : SIGMOID, "d_" + id));
		m_trainRnn->Connect(lastLayer, h, new DynamicWeight(m_layerWeights[i]));

		if (m_layerBias.size() > i)
			m_trainRnn->Connect(biases[i], h, new DynamicWeight(m_layerBias[i]));

		lastLayer = h;
	}
		
	for (int i = 0; i < m_maxLength - 1; ++i) 
	{
		std::string id = std::to_string(i+1);
		Layer* l = m_trainRnn->GetLayerByName("h_" + id);
		
		m_trainRnn->Connect(lastLayer, l, new DynamicWeight(m_H));
	}
	
	m_trainRnn->Optimize();
	m_ctxInputLayer = cmInput;
	return lastLayer;
}

DcgmI* DcgmI::LoadFromFile(const std::string& dirPath) {
	DcgmI* a = new DcgmI(true);

	CudaMatrix* Win = new CudaMatrix();
	CudaMatrix* Whh = new CudaMatrix();
	CudaMatrix* Wout = new CudaMatrix();
	CudaMatrix* H = new CudaMatrix();
	CudaMatrix* bhh = new CudaMatrix();
	CudaMatrix* bout = new CudaMatrix();

	Whh->LoadFromExternalBinary(dirPath + "\\w_hh.bin");
	std::cerr << Whh->GetRows() << " " << Whh->GetCols() << " " << Whh->Norm2() << std::endl;

	Win->LoadFromExternalBinary(dirPath + "\\w_in.bin");
	std::cerr << Win->GetRows() << " " << Win->GetCols() << " " << Win->Norm2() << std::endl;
	
	Wout->LoadFromExternalBinary(dirPath + "\\w_out.bin");
	std::cerr << Wout->GetRows() << " " << Wout->GetCols() << " " << Wout->Norm2() << std::endl;
	
	H->LoadFromExternalBinary(dirPath + "\\h.bin");
	std::cerr << H->GetRows() << " " << H->GetCols() << " " << H->Norm2() << std::endl;
	
	bhh->LoadFromExternalBinary(dirPath + "\\b_hh.bin");
	bout->LoadFromExternalBinary(dirPath + "\\b_out.bin");
		
	Config c = Config::ReadConfigFile(dirPath + "\\config");
	a->SetAct(c.GetString("act", "sigmoid"));

	std::cerr << "RnnActivation = " << c.GetString("act", "sigmoid") << std::endl;
	
	std::ifstream f(dirPath + "\\d_num.txt");
	int numLayers = 0;
	f >> numLayers;
	f.close();
	std::cerr << numLayers << std::endl;

	std::vector<CudaMatrix*> layerWeights;
	std::vector<CudaMatrix*> layerBias;
	std::vector<size_t> layerDims;

	for (int i = 0; i < numLayers; i++) {
		CudaMatrix* D = new CudaMatrix();
		D->LoadFromExternalBinary(dirPath + "\\d_" + std::to_string(i) + ".bin");
		std::cerr << D->GetName() << " " << D->GetRows() << " " << D->GetCols() << " " << D->Norm2() << std::endl;
		layerWeights.push_back(D);

		CudaMatrix* b = new CudaMatrix();
		b->LoadFromExternalBinary(dirPath + "\\b_" + std::to_string(i) + ".bin");
		std::cerr << b->GetName() << " " << b->GetRows() << " " << b->GetCols() << " " << b->Norm2() << std::endl;
		layerBias.push_back(b);
		
		layerDims.push_back(D->GetRows());
	}

	a->SetExternalInWeights(Win, Whh, Wout);
	a->SetContextRnnWeights(H);
	a->SetExternalBias(bhh, bout);
	a->SetExternalWeights(layerDims, layerWeights, layerBias);
	
	Vocabulary* o = new Vocabulary(dirPath + "\\vocab");
	RandomGenerator* g = new RandomGenerator(o);

	a->SetGenerator(g);
	a->SetVocabulary(o);
	return a;
}