#pragma once

#include "Fcgm.hpp"

class DcgmII : public Fcgm {
protected:
	CudaMatrix* m_ctxCInputs;
	std::vector<size_t> m_layerDims;
	std::vector<CudaMatrix*> m_layerWeights;
	std::vector<CudaMatrix*> m_layerBias;
	
	SparseInputLayer* m_ctxCInputLayer;

public:
	DcgmII() : Fcgm() {
		m_modelName = "DCGM-II";
		
		// Initialize inputs & outputs
		m_ctxCInputs = new CudaMatrix(1, 1, MatrixType::SPARSE);
		m_ctxCInputs->m_sparse->EnsureSpace(100*256);
		//
	}
	DcgmII(bool proprietaryMemory) : Fcgm(proprietaryMemory) {
		m_modelName = "DCGM-II";
		// Initialize inputs & outputs
		m_ctxCInputs = new CudaMatrix(1, 1, MatrixType::SPARSE);
		m_ctxCInputs->m_sparse->EnsureSpace(100*256);
		//
	}
	DcgmII(int nHid, int nOut) : Fcgm(nHid, nOut) {
		m_modelName = "DCGM-II";
		// Initialize inputs & outputs
		m_ctxCInputs = new CudaMatrix(1, 1, MatrixType::SPARSE);
		m_ctxCInputs->m_sparse->EnsureSpace(100*256);
		//
	}

	virtual ~DcgmII() {
		if (m_ctxCInputs) delete m_ctxCInputs;
	}
	
	void SetExternalWeights(const std::vector<size_t>& layerDims, 
			const std::vector<CudaMatrix*>& layerWeights,
			const std::vector<CudaMatrix*>& layerBias) {
		if (layerDims.size() == 0) {
			throw std::exception("Layer dimensions cannot be 0.");
		}

		m_layerDims = layerDims;
		m_nC = m_layerDims[m_layerDims.size()-1];
		m_layerWeights = layerWeights;
		m_layerBias = layerBias;
	}

	static DcgmII* LoadFromFile(const std::string& dirPath);
	
	void DeleteWeights() {
		if (m_H != nullptr) delete m_H;
		if (m_Whh != nullptr) delete m_Whh;
		if (m_Win != nullptr) delete m_Win;
		if (m_Wout != nullptr) delete m_Wout;
		for (int i = 0; i < m_layerWeights.size(); i++)
			if (m_layerWeights[i] != nullptr)
				delete m_layerWeights[i];
		for (int i = 0; i < m_layerBias.size(); i++)
			if (m_layerBias[i] != nullptr)
				delete m_layerBias[i];
	}
	
	void PrepareMatrices(const Batch<Triple>& batch, bool isTraining);
	virtual void SetCtxIO(int seqLen, int batchSize);
	virtual Layer* ComputeContext();
};