#include "DcgmIII.hpp"

// Here we build the whole network
// @Override from Rnn
void DcgmIII::BuildNetwork()
{
	HiddenLayer* lastHiddenLayer = nullptr;
	m_trainRnn = new NeuralNet();

	std::cerr << " Setting tau to " << m_tau << std::endl;
	m_trainRnn->SetBptt(m_Whh, m_tau);

	BiasLayer* bhh = m_trainRnn->AddLayer(new BiasLayer(m_batchSize, "bhh"));
	BiasLayer* bout = m_trainRnn->AddLayer(new BiasLayer(m_batchSize, "bout"));
	m_biasLayers.push_back(bhh);
	m_biasLayers.push_back(bout);
	
	for (int i = 0; i < m_maxLength - 1; ++i) 
	{
		std::string id = std::to_string(i+1);

		// set current input
		SparseInputLayer* x_next = m_trainRnn->AddLayer(new SparseInputLayer("x_" + id));
		HiddenLayer * h_next = m_trainRnn->AddLayer(new HiddenLayer(m_act, "h_" + id));
		if (lastHiddenLayer != nullptr) m_trainRnn->Connect(lastHiddenLayer, h_next, new DynamicWeight(m_Whh));

		/// Input connections
		m_trainRnn->Connect(x_next, h_next, new DynamicWeight(m_Win));

		NCELossLayer* yNce_t = m_trainRnn->AddLayer(new NCELossLayer(25, "pred_" + id));
		CrossEntropyLossLayer* yCe_t = m_trainRnn->AddLayer(new CrossEntropyLossLayer("pred_" + id));

		m_inputLayers.push_back(x_next);
		m_nceLayers.push_back(yNce_t);
		m_ceLayers.push_back(yCe_t);

		m_trainRnn->Connect(bhh, h_next, new DynamicWeight(m_Bhh));
		m_trainRnn->Connect(bout, yCe_t, new DynamicWeight(m_Bout));
		m_trainRnn->Connect(bout, yNce_t, new DynamicWeight(m_Bout));		
		lastHiddenLayer = h_next;
	}
}

Layer* DcgmIII::ComputeContext() {
	// Build the context vector
	m_ctxInputLayer = m_trainRnn->AddLayer(new SparseInputLayer(m_nIn, "m_i"));
	m_ctxCInputLayer = m_trainRnn->AddLayer(new SparseInputLayer(m_nIn, "c_i"));
	
	BiasLayer* bm = m_trainRnn->AddLayer(new BiasLayer(m_batchSize, "bm"));
	BiasLayer* bc = m_trainRnn->AddLayer(new BiasLayer(m_batchSize, "bc"));
	m_biasLayers.push_back(bm);
	m_biasLayers.push_back(bc);

	std::vector<BiasLayer*> biases;
	for (int i = 1; i < m_layerWeights.size(); i++)
	{
		std::string id = std::to_string(i);
		BiasLayer* b = m_trainRnn->AddLayer(new BiasLayer(m_batchSize, "b_" + id));
		m_biasLayers.push_back(b);
		biases.push_back(b);
	}
	
	// Continue
	Layer* lastLayer = nullptr;
	for (size_t i = 0; i < m_layerWeights.size(); i++) {
		std::string id = std::to_string(i);
		if (i == 0) 
		{
			HiddenLayer* mh = m_trainRnn->AddLayer(new HiddenLayer(ID, "m_h"));
			HiddenLayer* ch = m_trainRnn->AddLayer(new HiddenLayer(ID, "c_h"));

			m_trainRnn->Connect(m_ctxInputLayer, mh, new DynamicWeight(m_layerWeights[i]));
			m_trainRnn->Connect(m_ctxCInputLayer, ch, new DynamicWeight(m_layerWeights[i]));
			
			m_trainRnn->Connect(bm, mh, new DynamicWeight(m_layerBias[i]));
			m_trainRnn->Connect(bc, ch, new DynamicWeight(m_layerBias[i]));
			
			ConcatLayer* concatInputs = m_trainRnn->AddLayer(new ConcatLayer(2, ID, "concat_cm"));
			concatInputs->RegisterParent(ch);
			concatInputs->RegisterParent(mh);
			
			// Connect now
			m_trainRnn->Connect(mh, concatInputs, new FixedWeight());
			m_trainRnn->Connect(ch, concatInputs, new FixedWeight());
			lastLayer = concatInputs;
		}
		else 
		{
			HiddenLayer* h = m_trainRnn->AddLayer(new HiddenLayer(SIGMOID, "d_" + id));
			m_trainRnn->Connect(lastLayer, h, new DynamicWeight(m_layerWeights[i]));
			m_trainRnn->Connect(biases[i-1], h, new DynamicWeight(m_layerBias[i]));
			lastLayer = h;
		}
	}
		
	for (int i = 0; i < m_maxLength - 1; ++i)
	{
		std::string id = std::to_string(i+1);
		Layer* l = m_trainRnn->GetLayerByName("h_" + id);
		
		ConcatLayer* concatBeforeOutput = m_trainRnn->AddLayer(new ConcatLayer(2, ID, "concat_cm_" + id));
		concatBeforeOutput->RegisterParent(lastLayer);
		concatBeforeOutput->RegisterParent(l);
			
		// Connect the inputs of the concatenation layer
		m_trainRnn->Connect(lastLayer, concatBeforeOutput, new FixedWeight());
		m_trainRnn->Connect(l, concatBeforeOutput, new FixedWeight());
		
		// Connect the output now
		// HiddenLayer* deepO = m_trainRnn->AddLayer(new HiddenLayer(2, ReLU, "deep_o_" + id));
		// m_trainRnn->Connect(concatBeforeOutput, deepO, new DynamicWeight(m_H));
		m_trainRnn->Connect(concatBeforeOutput, m_nceLayers[i], new DynamicWeight(m_Wout));
		m_trainRnn->Connect(concatBeforeOutput, m_ceLayers[i], new DynamicWeight(m_Wout));
	}
	
	// m_trainRnn->Optimize();
	return lastLayer;
}

DcgmIII* DcgmIII::LoadFromFile(const std::string& dirPath) {
	DcgmIII* a = new DcgmIII(true);

	CudaMatrix* Win = new CudaMatrix();
	CudaMatrix* Whh = new CudaMatrix();
	CudaMatrix* Wout = new CudaMatrix();
	CudaMatrix* H = new CudaMatrix();
	CudaMatrix* bhh = new CudaMatrix();
	CudaMatrix* bout = new CudaMatrix();

	Whh->LoadFromExternalBinary(dirPath + "\\w_hh.bin");
	std::cerr << Whh->GetRows() << " " << Whh->GetCols() << " " << Whh->Norm2() << std::endl;

	Win->LoadFromExternalBinary(dirPath + "\\w_in.bin");
	std::cerr << Win->GetRows() << " " << Win->GetCols() << " " << Win->Norm2() << std::endl;
	
	Wout->LoadFromExternalBinary(dirPath + "\\w_out.bin");
	std::cerr << Wout->GetRows() << " " << Wout->GetCols() << " " << Wout->Norm2() << std::endl;

	//H->LoadFromExternalBinary(dirPath + "\\h.bin");
	//std::cerr << H->GetRows() << " " << H->GetCols() << " " << H->Norm2() << std::endl;
	
	bhh->LoadFromExternalBinary(dirPath + "\\b_hh.bin");
	bout->LoadFromExternalBinary(dirPath + "\\b_out.bin");
		
	Config c = Config::ReadConfigFile(dirPath + "\\config");
	a->SetAct(c.GetString("act", "sigmoid"));

	std::cerr << "RnnActivation = " << c.GetString("act", "sigmoid") << std::endl;
	
	std::ifstream f(dirPath + "\\d_num.txt");
	int numLayers = 0;
	f >> numLayers;
	f.close();
	std::cerr << numLayers << std::endl;

	std::vector<CudaMatrix*> layerWeights;
	std::vector<CudaMatrix*> layerBias;
	std::vector<size_t> layerDims;

	for (int i = 0; i < numLayers; i++) {
		CudaMatrix* D = new CudaMatrix();
		D->LoadFromExternalBinary(dirPath + "\\d_" + std::to_string(i) + ".bin");
		std::cerr << D->GetName() << " " << D->GetRows() << " " << D->GetCols() << " " << D->Norm2() << std::endl;
		layerWeights.push_back(D);

		CudaMatrix* b = new CudaMatrix();
		b->LoadFromExternalBinary(dirPath + "\\b_" + std::to_string(i) + ".bin");
		std::cerr << b->GetName() << " " << b->GetRows() << " " << b->GetCols() << " " << b->Norm2() << std::endl;
		layerBias.push_back(b);
		
		layerDims.push_back(D->GetRows());
	}

	a->SetExternalInWeights(Win, Whh, Wout);
	a->SetContextRnnWeights(H);
	a->SetExternalBias(bhh, bout);
	a->SetExternalWeights(layerDims, layerWeights, layerBias);
	
	Vocabulary* o = new Vocabulary(dirPath + "\\vocab");
	RandomGenerator* g = new RandomGenerator(o);

	a->SetGenerator(g);
	a->SetVocabulary(o);
	return a;
}