#pragma once

#include "NnetLib.hpp"
#include "Rnn.hpp"
#include "GradientUpdater.hpp"
#include "GradientVisitor.hpp"
#include "Random.hpp"
#include "AbstractReader.hpp"

class Fcgm : public Rnn {
private:
	CudaMatrix* m_I;

protected:
	CudaMatrix* m_H;
	size_t m_nC;     // units of the context vector

	CudaMatrix* m_ctxInputs;
	SparseInputLayer* m_ctxInputLayer;

public:
	Fcgm();
	Fcgm(bool proprietaryMemory);
	Fcgm(int nHid, int nOut);
	Fcgm(const Fcgm& deep_copy) { NOT_IMPLEMENTED; }
	~Fcgm(void) {
		if (m_ctxInputs) delete m_ctxInputs;
	}
	
	void SetContextRnnWeights(CudaMatrix* H) {
		m_H = H;
	}

	void SetExternalWeights(CudaMatrix* I) {
		m_I = I;
		m_nC = I->GetRows();
	}

	void CleanNetwork(NeuralNet& rnn, bool cleanError=false);
	
	virtual Layer* ComputeContext();

	float Evaluate(const Batch<Triple>& batch, bool stochastic);
	float Run(const Batch<Triple>& batch, bool isTraining);
	
	virtual void PrepareMatrices(const Batch<Triple>& batch, bool isTraining);

	std::vector<int> Generate(const Triple& cmr);
	static Fcgm* LoadFromFile(const std::string& dirPath);
	
	void DeleteWeights() {
		if (m_I != nullptr) delete m_I;
		if (m_H != nullptr) delete m_H;
		if (m_Whh != nullptr) delete m_Whh;
		if (m_Win != nullptr) delete m_Win;
		if (m_Wout != nullptr) delete m_Wout;
	}

protected:
	virtual void SetCtxIO(int seqLen, int batchSize);
};