#include "LengthBatchedReader.hpp"

struct SortSecond {
	bool operator()(const std::pair<int,int> &left, const std::pair<int,int> &right) {
		return left.second < right.second;
	}
};

bool SequenceBatchedReader::ReadNextBatch() {
	bool stillRead = false;

	std::string line;
	int batchSize = 0;
	Batch<Sequence> wi;

	while (std::getline(readerStream, line)) {
		Sequence seq;
		std::stringstream ss(line);
		std::string tok;

		while (ss >> tok) 
			seq.push_back(std::atoi(tok.c_str()));

		if (seq.size() > 0) {
			wi.data.push_back(seq);
			wi.m_size = ++batchSize;
		}

		if (batchSize >= m_mbSize) {
			m_read += batchSize;
			stillRead = true;
			break;
		}
	}
	
	std::shuffle(wi.data.begin(), wi.data.end(), rd);
	
	std::vector<Batch<Sequence>> q;
	q.reserve(batchSize);

	if (batchSize == 0)
		return false;

	std::vector<std::list<int>> lengthIds(100); 
	for (int i = 0; i < batchSize; i++) {
		lengthIds[wi.data[i].size()].push_back(i);
	}
	
	for (int i = 0; i < lengths.size(); i++)
	{
		int lengthIdx = lengths[i];
		int numSize = lengthIds[lengthIdx].size();
		if (numSize == 0) continue;
		
		while (lengthIds[lengthIdx].size() > 0)
		{
			Batch<Sequence> subBatch;
					
			while (subBatch.m_size < BUNCH_SIZE && lengthIds[lengthIdx].size() > 0)
			{
				subBatch.data.push_back(wi.data[lengthIds[lengthIdx].front()]);		
				subBatch.m_size++;

				lengthIds[lengthIdx].pop_front();
			}

			// Enqueue item
			q.push_back(subBatch);
		}
	}

	std::shuffle(q.begin(), q.end(), rd);
	for (int i = 0; i < q.size(); i++)
		m_queue->Push(q[i]);

	return stillRead;
}

bool TripleBatchedReader::ReadNextBatch() {
	bool stillRead = false;

	std::string line;
	int batchSize = 0;
	Batch<Triple> wi;

	while (std::getline(readerStream, line)) {
		Triple t; t.reserve(3);

		// Parse sequences
		std::istringstream ss(line);
		std::string sequence;

		while (std::getline(ss, sequence, '\t')) {
			// Tokenize it
			std::stringstream tokenizer(sequence);
			
			Sequence seq;
			std::string tok;
			while (tokenizer >> tok) 
				seq.push_back(std::atoi(tok.c_str()));
			
			if (seq.size() > 0) {
				t.push_back(seq);
			}
		}
		
		if (t.size() != 3) {
			// Do not load this triple but we do it silently..
			// never do errors!
			continue;
		}

		wi.data.push_back(t);
		wi.m_size = ++batchSize;

		if (batchSize >= m_mbSize) {
			m_read += batchSize;
			stillRead = true;
			break;
		}
	}
	
	std::shuffle(wi.data.begin(), wi.data.end(), rd);
	
	// All the batches
	std::vector<Batch<Triple>> q;
	q.reserve(batchSize);

	if (batchSize == 0)
		return false;

	std::vector<std::list<int>> lengthIds(100); 
	for (int i = 0; i < batchSize; i++) {
		// Target sentence size
		lengthIds[wi.data[i][2].size()].push_back(i);
	}
	
	for (int i = 0; i < lengths.size(); i++)
	{
		int lengthIdx = lengths[i];
		int numSize = lengthIds[lengthIdx].size();
		if (numSize == 0) continue;
		
		while (lengthIds[lengthIdx].size() > 0)
		{
			Batch<Triple> subBatch;
		
			while (subBatch.m_size < BUNCH_SIZE && lengthIds[lengthIdx].size() > 0)
			{
				subBatch.data.push_back(wi.data[lengthIds[lengthIdx].front()]);		
				subBatch.m_size++;

				lengthIds[lengthIdx].pop_front();
			}

			// Enqueue item
			q.push_back(subBatch);
		}
	}
	
	std::shuffle(q.begin(), q.end(), rd);
	for (int i = 0; i < q.size(); i++)
		m_queue->Push(q[i]);

	return stillRead;
}