#pragma once

#include <string>
#include <iostream>
#include <fstream>
#include "DcgmI.hpp"
#include "DcgmII.hpp"
#include "DcgmIII.hpp"
#include "DcgmIV.hpp"
#include "Fcgm.hpp"
#include "AbstractModel.hpp"

class ModelLoader
{
public:
	static AbstractModel* LoadFromFile(const std::string& dirPath) 
	{
		std::cout << "--------------- MODEL LOADER ---------------" << std::endl;
		std::ifstream f(dirPath + "\\modelType");
		if (!f.good())
		{
			throw std::exception("Model dir not found!");
		}
		
		std::string modelType = "";
		f >> modelType;
		f.close();

		std::cout << "- Detected " << modelType << std::endl;
		
		AbstractModel* m = nullptr;
		if (modelType == "DCGM-I")
			m = DcgmI::LoadFromFile(dirPath);
		else if (modelType == "DCGM-II")
			m = DcgmII::LoadFromFile(dirPath);
		else if (modelType == "DCGM-III")
			m = DcgmIII::LoadFromFile(dirPath);
		else if (modelType == "DCGM-IV")
			m = DcgmIV::LoadFromFile(dirPath);
		else if (modelType == "FCGM")			
			m = Fcgm::LoadFromFile(dirPath);

		std::cout << "Vocabulary size = " << m->GetVocabulary()->vocab_size << std::endl;
		std::cout << "---------------------------------------------" << std::endl;
		return m;
	}
};