#pragma once

#include "Vocabulary.hpp"
#include <random>
#include <map>


class RandomGenerator {
private:
	int size_;
	std::default_random_engine rd;
	std::discrete_distribution<> unigram;
	std::vector<float> log_probs_;

public:
	RandomGenerator() : size_(0) {
		rd = std::default_random_engine(1);
	};
	RandomGenerator(int size) : size_(size) {
		rd = std::default_random_engine(1);
	};
	RandomGenerator(Vocabulary* o) : size_(0) {
		rd = std::default_random_engine(1);
		CreateFromVocabulary(*o);	
	};

	void CreateFromVocabulary(const Vocabulary& v) 
	{
		size_ = (int) v.vocab_size;
		log_probs_.reserve(size_);
		unigram = std::discrete_distribution<>(v.vocab_size, 0, (float) v.vocab_size, 
			[&](int x)->long long { 
				float prob = (float) (log((float)v.vocab[x].cn) - log((float)v.train_words));
				log_probs_.push_back(prob);
				return (long long) v.vocab[x].cn;
		});
	}

	static void Create(RandomGenerator& g, int size) {
		g.size_ = size;
		g.log_probs_.reserve(g.size_);
		// uniform prob
		float prob = (float) (log(1.0)-log(g.size_));
		g.unigram = std::discrete_distribution<>(g.size_, 0, (float) g.size_, 
			[&](int x)->long long { 
				g.log_probs_.push_back(prob);
				return (long long) 1; 
			}
		);
	}
	
	static void Create(RandomGenerator& g, Vocabulary& v) {
		g.CreateFromVocabulary(v);
	}

	// sample_noise - we sample the real noise here!! 
	// t - is the target, we don't want that?
	// N - how many noise samples?
	std::vector<int> SampleNoise(int N, int t=-1) {
		std::vector<int> samples;
		for (int r = 0; r < N; r++) {
			int index = unigram(rd);
			if (index == t) index = (index + 1) % size_;
			samples.push_back(index);
		}

		return samples;
	}
	
	// get_probs - get the probabilities of each noise sample under the unigram distribution
	float GetLogProb(int id) {
		if (id >= size_) throw new std::exception("Random_get_prob: index out of range");
		return log_probs_[id];
	}
};