#pragma once

#include "DcgmII.hpp"

class Rcgm : public DcgmII 
{
	CudaMatrix* m_Wshh;
	CudaMatrix* m_Wsin;
	CudaMatrix* m_bshh;

	std::vector<SparseInputLayer*> m_cInputLayers;
	std::vector<SparseInputLayer*> m_mInputLayers;
	std::vector<NCELossLayer*> m_nceLayers;
	std::vector<CrossEntropyLossLayer*> m_ceLayers;

private:
	Layer* BuildRnn(Layer* contextLayer, std::string seqName);
	void BuildNetwork();
	
	void SetIO(int seqLen, int batchSize, 
		const CudaMatrix& inputs,
		const CudaMatrix& outputs,
		bool training=true);
	void SetCtxIO(int seqLen, int batchSize);

	virtual void PrepareMatrices(const Batch<Triple>& batch, bool isTraining);

public:
	Rcgm() : DcgmII() {}
	Rcgm(int nHid, int nOut) : DcgmII(nHid, nOut) {}
	~Rcgm() {
		if (m_proprietaryMemory)
		{
			if (m_Wshh) delete m_Wshh; 
			if (m_Wsin) delete m_Wsin; 
			if (m_bshh) delete m_bshh;
		}
	}

	void SetSentenceRnnWeights(CudaMatrix* WShh, CudaMatrix* WSin, CudaMatrix* bshh) {
		m_Wsin = WSin;
		m_Wshh = WShh;
		m_bshh = bshh;
	}

	static Rcgm* LoadFromFile(const std::string& dirPath);

	float Run(const Batch<Triple>& batch, bool isTraining);
};