#pragma once

#include "DcgmTrainer.hpp"
#include "print.hpp"
#include "Rcgm.hpp"
#include <chrono>

class RcgmTrainer : public FcgmTrainer {
protected:
	int m_nsHid;
	CudaMatrix WShh;
	CudaMatrix WSin;
	CudaMatrix bshh;

public:
	RcgmTrainer(const std::string& outputPath) : FcgmTrainer(outputPath) {}

	void InitializeRcgmWeights(int nIn, int nHid, int nOut) { 
		long seed = 0;

		InitializeRnnWeights(nIn, nHid, nOut, seed);

		m_nsHid = m_nHid;

		std::cerr << "Initializing model RCGM-" << m_nIn << ":" << m_nHid << ":" << m_nOut << ":" << m_nsHid;
		
		seed++;
		WSin.SetName("S_in");
		WSin.SetSeed(seed);
		WSin.Resize(m_nHid, m_nsHid);
		WSin.GaussianRandomFill(0.f, 0.1f);

		seed++;
		WShh.SetName("S_hh");
		WShh.Resize(m_nsHid, m_nsHid);
		WShh.SetSeed(seed); 
		WShh.GaussianRandomFill(0.f, 0.1f);

		bshh.Resize(1, m_nsHid);
		bshh.SetName("bs_hh");

		seed++;
		H.SetSeed(seed);
		H.Resize(m_nsHid, m_nHid);
		H.SetName("H");
		H.GaussianRandomFill(0.f, 0.1f);
	}

	void InitializeRcgmWeights(std::string& pretrainedRnn) { 
		Whh.LoadFromExternalBinary(pretrainedRnn + "\\w_hh.bin");
		Win.LoadFromExternalBinary(pretrainedRnn + "\\w_in.bin");
		Wout.LoadFromExternalBinary(pretrainedRnn + "\\w_out.bin");

		bhh.LoadFromExternalBinary(pretrainedRnn + "\\b_hh.bin");
		bout.LoadFromExternalBinary(pretrainedRnn + "\\b_out.bin");
		
		m_nHid = Whh.GetRows();
		m_nIn = Wout.GetCols();
		m_nOut = m_nIn;
		m_nsHid = m_nHid/2;

		std::cerr << "Initializing model RCGM-" << m_nIn << ":" << m_nHid << ":" << m_nOut << ":" << m_nsHid;
		
		long seed = 1;
		
		seed++;
		WSin.SetName("S_in");
		WSin.SetSeed(seed);
		WSin.Resize(m_nHid, m_nsHid);
		WSin.GaussianRandomFill(0.f, 0.01f);

		seed++;
		WShh.SetName("S_hh");
		WShh.Resize(m_nsHid, m_nsHid);
		WShh.SetSeed(seed); 
		WShh.GaussianRandomFill(0.f, 0.01f);

		bshh.Resize(1, m_nsHid);
		bshh.SetName("bs_hh");

		seed++;
		H.SetSeed(seed);
		H.Resize(m_nsHid, m_nHid);
		H.SetName("H");
		H.GaussianRandomFill(0.f, 0.01f);
	}

	void DoJob(AbstractTrainer<Batch<Triple>, TripleBatchedReader>& parentCaller, 
		ThreadedQueue<Batch<Triple>>& workingQueue,
		bool isTraining=false);
	void WriteModel();
	void PrintStatus(bool training=true);
	
	~RcgmTrainer() {}
};