#include "Rnn.hpp"

Rnn::Rnn(bool proprietaryMemory) : AbstractModel(proprietaryMemory) {
	m_modelName = "RNN";
	m_nIn = 0;
	m_act = SIGMOID;
	m_nOut = 0;
	m_nHid = 0;
	m_tau = 7;	
	m_Bout = nullptr;
	m_Bhh = nullptr;
	m_updater = nullptr;
	m_generator = nullptr;
	m_Whh = nullptr;
	m_dIn = nullptr;
	m_dOut = nullptr;
	m_Wout = nullptr;
	m_Win = nullptr;
	m_trainRnn = nullptr;
	m_batchSize = 100;
	m_maxLength = 100;

	m_rnnInputs = new CudaMatrix(1, 1, MatrixType::SPARSE);
	m_rnnInputs->m_sparse->EnsureSpace(100*256);

	m_rnnOutputs = new CudaMatrix(1, 1, MatrixType::SPARSE);
	m_rnnOutputs->m_sparse->EnsureSpace(100*256*26);
}

Rnn::Rnn() : AbstractModel() {
	m_modelName = "RNN";
	m_nIn = 0;
	m_act = SIGMOID;
	m_nOut = 0;
	m_nHid = 0;
	m_tau = 7;	
	m_Bout = nullptr;
	m_Bhh = nullptr;
	m_updater = nullptr;
	m_generator = nullptr;
	m_Whh = nullptr;
	m_dIn = nullptr;
	m_dOut = nullptr;
	m_Wout = nullptr;
	m_Win = nullptr;
	m_trainRnn = nullptr;
	m_batchSize = 100;
	m_maxLength = 100;

	m_rnnInputs = new CudaMatrix(1, 1, MatrixType::SPARSE);
	m_rnnInputs->m_sparse->EnsureSpace(100*256);

	m_rnnOutputs = new CudaMatrix(1, 1, MatrixType::SPARSE);
	m_rnnOutputs->m_sparse->EnsureSpace(100*256*26);
	//
}
Rnn::Rnn(int nHid, int nOut) : AbstractModel() {
	m_nIn = 0;
	m_modelName = "RNN";
	m_nOut = nOut;
	m_nHid = nHid;
	m_Bout = nullptr;
	m_Bhh = nullptr;
	m_act = SIGMOID;
	m_updater = nullptr;
	m_generator = nullptr;
	m_Whh = nullptr;
	m_Wout = nullptr;
	m_dIn = nullptr;
	m_dOut = nullptr;
	m_Win = nullptr;
	m_tau = 7;
	m_trainRnn = nullptr;
	m_batchSize = 100;
	m_maxLength = 100;

	m_rnnInputs = new CudaMatrix(1, 1, MatrixType::SPARSE);
	m_rnnInputs->m_sparse->EnsureSpace(100*256);

	m_rnnOutputs = new CudaMatrix(1, 1, MatrixType::SPARSE);
	m_rnnOutputs->m_sparse->EnsureSpace(100*256*26);
	//
}

// Here we build the whole network
// avoiding to reallocate space for each new sentence 
// that comes in.
void Rnn::BuildNetwork()
{
	HiddenLayer* lastHiddenLayer = nullptr;
	m_trainRnn = new NeuralNet();

	std::cerr << " Setting tau to " << m_tau << std::endl;
	m_trainRnn->SetBptt(m_Whh, m_tau);

	BiasLayer* bhh = m_trainRnn->AddLayer(new BiasLayer(m_batchSize, "bhh"));
	BiasLayer* bout = m_trainRnn->AddLayer(new BiasLayer(m_batchSize, "bout"));
	m_biasLayers.push_back(bhh);
	m_biasLayers.push_back(bout);
	
	for (int i = 0; i < m_maxLength - 1; ++i) 
	{
		std::string id = std::to_string(i+1);

		// set current input
		SparseInputLayer* x_next = m_trainRnn->AddLayer(new SparseInputLayer("x_" + id));
		HiddenLayer * h_next = m_trainRnn->AddLayer(new HiddenLayer(m_act, "h_" + id));
		if (lastHiddenLayer != nullptr) m_trainRnn->Connect(lastHiddenLayer, h_next, new DynamicWeight(m_Whh));

		/// Deep input connections
		if (m_dIn != nullptr)
		{
			HiddenLayer* x_deepIn = m_trainRnn->AddLayer(new HiddenLayer(m_act, "xd_" + id));
			m_trainRnn->Connect(x_next, x_deepIn, new DynamicWeight(m_Win));
			m_trainRnn->Connect(x_deepIn, h_next, new DynamicWeight(m_dIn));
		}
		else
		{
			m_trainRnn->Connect(x_next, h_next, new DynamicWeight(m_Win));
		}

		NCELossLayer* yNce_t = m_trainRnn->AddLayer(new NCELossLayer(25, "pred_" + id));
		CrossEntropyLossLayer* yCe_t = m_trainRnn->AddLayer(new CrossEntropyLossLayer("pred_" + id));

		m_inputLayers.push_back(x_next);
		m_nceLayers.push_back(yNce_t);
		m_ceLayers.push_back(yCe_t);

		m_trainRnn->Connect(h_next, yNce_t, new DynamicWeight(m_Wout));
		m_trainRnn->Connect(h_next, yCe_t, new DynamicWeight(m_Wout));
				
		m_trainRnn->Connect(bhh, h_next, new DynamicWeight(m_Bhh));
		m_trainRnn->Connect(bout, yCe_t, new DynamicWeight(m_Bout));
		m_trainRnn->Connect(bout, yNce_t, new DynamicWeight(m_Bout));		
		lastHiddenLayer = h_next;
	}
}

std::vector<int> Rnn::Generate(const Sequence& context) { NOT_IMPLEMENTED; }
float Rnn::Evaluate(const Sequence& sequence, bool stoch) { NOT_IMPLEMENTED; }
float Rnn::Evaluate(const Sequence& ctx, const Sequence& sequence, bool stoch) { NOT_IMPLEMENTED; }
void AddStep(const Sequence& sequence, size_t step, NeuralNet& rnn, bool training=true) { NOT_IMPLEMENTED; }
float Train(int seqLen, int batchSize, const CudaMatrix& rnnInputs, const CudaMatrix& rnnOutputs) { NOT_IMPLEMENTED; }

void Rnn::CleanNetwork(NeuralNet& rnn, bool cleanActivations) {
	ResetVisitor v;
	v.cleanError_ = true;
	v.cleanActivation_ = cleanActivations;
	rnn.Visit(v);
}

Rnn* Rnn::LoadFromFile(const std::string& dirPath) {
	Rnn* a = new Rnn();

	CudaMatrix* Win = new CudaMatrix();
	CudaMatrix* Whh = new CudaMatrix();
	CudaMatrix* Wout = new CudaMatrix();
	CudaMatrix* bout = new CudaMatrix();
	CudaMatrix* bhh = new CudaMatrix();

	Whh->LoadFromExternalBinary(dirPath + "\\w_hh.bin");
	std::cerr << Whh->GetRows() << " " << Whh->GetCols() << " " << Whh->Norm2() << std::endl;
	
	Win->LoadFromExternalBinary(dirPath + "\\w_in.bin");
	std::cerr << Win->GetRows() << " " << Win->GetCols() << " " << Win->Norm2() << std::endl;
	
	Wout->LoadFromExternalBinary(dirPath + "\\w_out.bin");
	std::cerr << Wout->GetRows() << " " << Wout->GetCols() << " " << Wout->Norm2() << std::endl;
	
	bout->LoadFromExternalBinary(dirPath + "\\b_out.bin");
	bhh->LoadFromExternalBinary(dirPath + "\\b_in.bin");

	a->SetExternalBias(bhh, bout);
	a->SetExternalInWeights(Win, Whh, Wout);
	return a;
}

/// PrepareMatrices - we stock all the necessary information for the current sequence
///					  on the gpu.
/// [in] - subBatch, inputs and outputs (time ordered)
void Rnn::PrepareMatrices(const Batch<Sequence>& batch, bool isTraining)
{
	// Find the max length
	int batchSize = (int) batch.data.size();
	int sentLen = (int) batch.data[0].size();

	std::vector<int> ctxInputRows; 
	std::vector<float> ctxInputValues;
	std::vector<int> ctxInputCols;

	std::vector<int> inputRows; 
	std::vector<float> inputValues;
	std::vector<int> inputCols;

	std::vector<int> outputRows;
	std::vector<int> outputCols;
	std::vector<float> outputValues;

	for (int j = 0; j < sentLen - 1; j++)
	{
		for (int i = 0; i < batchSize; ++i) {
			// Take the last sequence
			const Sequence& seq = batch.data[i];

			int input = seq[j];
			int output = seq[j+1];

			// [batch_n, token_id] = 1
			inputRows.push_back(i);
			inputCols.push_back(input);
			inputValues.push_back(1);

			if (!isTraining)
			{
				/// If we evaluate we only push the real output value
				outputRows.push_back(i);
				outputCols.push_back(output);	
				outputValues.push_back(1);
			}
			else
			{
				std::vector<int> noise = m_generator->SampleNoise(25, output);
				for (int x = 0; x < noise.size(); x++) 
				{
					float logPn = m_generator->GetLogProb(noise[x]);

					outputRows.push_back(i);
					outputCols.push_back(noise[x]);
					outputValues.push_back(logPn);
				}

				// Push the target
				outputRows.push_back(i);
				outputCols.push_back(output);
				outputValues.push_back(-m_generator->GetLogProb(output));
			}
		}
	}

	int ctxInputsNnz = (int) ctxInputRows.size();	
	int inputNnz = (int) inputRows.size();
	int outputNnz = (int) outputRows.size();

	/// We set the values
	m_rnnInputs->SetData(1, 1, inputNnz, inputRows.data(), inputCols.data(), inputValues.data());
	m_rnnOutputs->SetData(1, 1, outputNnz, outputRows.data(), outputCols.data(), outputValues.data());
}

void Rnn::SetIO(int seqLen, int effectiveBatchSize, 
				const CudaMatrix& inputs, 
				const CudaMatrix& outputs,
				bool isTraining)
{
	const std::vector<InputLayer*>& I = m_trainRnn->Inputs();

	SparseCudaMatrix* outputSparse = outputs.GetSparseMatrix();
	SparseCudaMatrix* inputSparse = inputs.GetSparseMatrix();

	for (int i = 0; i < seqLen - 1; ++i) {
		int iBase = i*effectiveBatchSize;
		int oBase = 26*iBase;

		if (isTraining)
			m_nceLayers[i]->SetOutput(effectiveBatchSize, m_nOut, 26 * effectiveBatchSize, 
			outputSparse->m_cooRow + oBase,
			outputSparse->m_cooCol + oBase, 
			outputSparse->m_cooVal + oBase, nullptr, true, true);
		else
			m_ceLayers[i]->SetOutput(effectiveBatchSize, m_nOut, effectiveBatchSize, 
			outputSparse->m_cooRow + iBase,
			outputSparse->m_cooCol + iBase,
			outputSparse->m_cooVal + iBase, true, true);
		m_inputLayers[i]->SetInput(effectiveBatchSize, m_nIn, effectiveBatchSize, 
			inputSparse->m_cooRow + iBase,
			inputSparse->m_cooCol + iBase, 
			inputSparse->m_cooVal + iBase, true, true);
	}

	if (m_biasLayers.size() != 0)
	{
		m_biasLayers[0]->SetInput(effectiveBatchSize);
		m_biasLayers[1]->SetInput(effectiveBatchSize);
	}
}

float Rnn::Run(const Batch<Sequence>& batch, bool isTraining) {	
	float error = 0;
	bool networkChanged = true;
	
	int batchSize = (int) batch.data.size();
	int sentLen = (int) batch.data[0].size();

	if (m_trainRnn == nullptr) BuildNetwork();

	PrepareMatrices(batch, isTraining);
	SetIO(sentLen, batchSize, *m_rnnInputs, *m_rnnOutputs, isTraining);
	
	// Run through the RNN
	for (int i = 0; i < sentLen - 1; ++i)
	{
		CleanNetwork(*m_trainRnn, networkChanged);
		networkChanged = false;

		LossLayer* toEvaluate = nullptr;
		if (isTraining) toEvaluate = m_nceLayers[i];
		else toEvaluate = m_ceLayers[i];

		CudaMatrix* output = m_trainRnn->Forward(toEvaluate);
		error += m_trainRnn->GetLoss(toEvaluate);

		if (isTraining)
		{
			m_trainRnn->Backward(toEvaluate);
			m_trainRnn->Visit(v);
			m_updater->UpdateParameters(v);
			networkChanged = true; // trigger to clean the rnn*/
		}
	}

	return error;
}