
#pragma once

#include "AbstractTrainer.hpp"
#include "GradientUpdater.hpp"
#include "SequenceReader.hpp"
#include "LengthBatchedReader.hpp"
#include "Rnn.hpp"
#include "CudaMatrix.hpp"
#include <chrono>

class RnnTrainer : public AbstractTrainer<Batch<Sequence>, SequenceBatchedReader> {
protected:
	int m_tau;
	int m_nIn; 
	int m_nHid; 
	int m_nOut;
	
	Activation m_act;

	CudaMatrix Win;
	CudaMatrix Wout;
	CudaMatrix Whh;

	CudaMatrix bhh;
	CudaMatrix bout;

private:
	void PrepareMatrices(int batchSize, const Batch<Sequence>& subBatch, CudaMatrix& inputs, CudaMatrix& outputs, bool evaluate);

public:
	RnnTrainer(const std::string& outputPath) : AbstractTrainer(outputPath) {
		m_act = SIGMOID;
		m_tau = 7;
	}

	void InitializeWeights(int nIn, int nHid, int nOut) { 
		m_nIn = nIn;
		m_nHid = nHid;
		m_nOut = nOut;
		
		Win.Resize(m_nIn,m_nHid);
		Win.SetName("W_in");
		
		Wout.Resize(m_nHid,m_nOut);
		Wout.SetName("W_out");
		
		Whh.Resize(m_nHid, m_nHid);
		Whh.SetName("W_hh");
		
		bhh.Resize(1, m_nHid);
		bhh.SetName("b_hh");
		
		bout.Resize(1, m_nOut);
		bout.SetName("b_out");
				
		Win.GaussianRandomFill(0.f, 0.1f);
		Wout.GaussianRandomFill(0.f, 0.1f);
		//Whh.OrthogonalFill(0.f, 0.1f); // NAACL
		Whh.GaussianRandomFill(0.f, 0.1f);
	}
	
	void SetActivation(const std::string& act) 
	{ 
		if (act == "sigmoid") m_act = SIGMOID; 
		else if (act == "tanh") m_act = TANH;
		else if (act == "relu") m_act = ReLU;
	}

	void SetTau(int tau) { m_tau = tau; }
	void DoJob(AbstractTrainer<Batch<Sequence>, SequenceBatchedReader>& parentCaller, 
		ThreadedQueue<Batch<Sequence>>& workingQueue,
		bool isTraining=false);
	void WriteModel();

	void PrintStatus(bool training=true);

	~RnnTrainer() {
		if (m_upd) delete m_upd;	
	}
};