#include "TGTReader.hpp"

#include <sstream>

bool TGTReader::ReadNextCandidate(HeadRecord& x) {
	if (m_startRead == false) {
		m_fileH = std::ifstream(m_fileName);
		if (!m_fileH.good()) {
			std::cerr << m_fileName << std::endl;
			std::cerr << "The specified nbest file was not found" << std::endl;
			exit(-1);	
		}

		m_ctxH = std::ifstream(m_fileName + ".c");
		m_startRead = true;
	}

	bool referenceFound = false;
	bool eof = false;
	std::string line = "";
	
	x = HeadRecord();
	while (std::getline(m_fileH, line)) {
		if (line == "" && x.rlist.size() > 0) {
			referenceFound = true;
			break;
		}

		// This should be the source
		if (x.mtext == "") {
			// Read also the line from m_ctxH
			if (!std::getline(m_ctxH, x.ctext))
			{
				throw std::exception("Length mismatch between context file and tgt file!");
			}

			x.mtext = line;
		}

		size_t pos = 0;
		if ((pos=line.find("\\\REF")) != std::string::npos) {
			x.rtext  = line.substr(pos+4);
			continue;
		}

		if ((pos=line.find("@@H")) != std::string::npos) {
			line = line.substr(pos+3);
			std::stringstream ss(line);
			std::string featureName;
			while (std::getline(ss, featureName, ',')) {
				x.featureNames.push_back(featureName);
			}
		}

		if (line.find("@@Q") != std::string::npos) {
			// Start parsing tail records
			while (1)
			{
				std::getline(m_fileH, line);
				// We reached the end
				if (line.find("@@===") != std::string::npos)
					break;

				TailRecord t; t.rtext = line;
				
				std::getline(m_fileH, line);
				pos = 0;
				
				if ((pos = line.find("@@V")) != std::string::npos) {
					line = line.substr(pos+3);
					std::stringstream ss(line);
					std::string featureValue;
					while (std::getline(ss, featureValue, ',')) {
						t.featureValues.push_back(std::atof(featureValue.c_str()));
					}
				} else 
					throw std::exception("TGT format error!");
				
				x.rlist.push_back(t);
			}
		}

		if (line.find("-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-") != std::string::npos) {
			continue;		
		}
	}

	return referenceFound;
}