//
//  Activation.hpp
//  Nnet
//
//  Created by Alessandro Sordoni on 2014-04-23.
//  Copyright (c) 2014 Alessandro Sordoni. All rights reserved.
//

#pragma once

#include "CudaMatrix.hpp"

typedef std::string Activation;
const std::string ID = "id";
const std::string SOFTMAX = "softmax";
const std::string TANH = "tanh";
const std::string SIGMOID = "sigmoid";
const std::string ReLU = "relu";

class ActivationFn {
public:
	virtual void f(CudaMatrix* p) = 0;
    virtual void df(const CudaMatrix* activation, CudaMatrix* error) = 0;
                      
    virtual ~ActivationFn() {};
};

class IdFn : public ActivationFn {
public:
    void f(CudaMatrix* p) {}
	void df(const CudaMatrix* p, CudaMatrix* ap) {}
    ~IdFn() {};
};

class SigmoidFn : public ActivationFn {
public:
    void f(CudaMatrix* p) {
		p->InPlaceSigmoid();
    }
    
	void df(const CudaMatrix* activation, CudaMatrix* error) {
		error->InPlaceSigmoidDerivative(*activation);
    }
    
    ~SigmoidFn() {}
};

class TanhFn : public ActivationFn {
public:
    void f(CudaMatrix* p) {
		p->InPlaceTanh();
	}
	void df(const CudaMatrix* activation, CudaMatrix* error) {
		error->InPlaceTanhDerivative(*activation);
	}
    ~TanhFn() {}
};

class ReLUFn : public ActivationFn {
public:
    void f(CudaMatrix* p) {
		p->InPlaceRelu();
    }

    void df(const CudaMatrix* activation, CudaMatrix* error) {
		error->InPlaceReluDerivative(*activation);
    }

    ~ReLUFn() {}
};

class ExpFn : public ActivationFn {
public:
	void f(CudaMatrix* p) {
    }

    void df(const CudaMatrix* p, CudaMatrix* ap) {
    }

    ~ExpFn() {}
};

class SoftmaxFn : public ActivationFn {
public:
    void f(CudaMatrix* p) {
		p->InPlaceSoftmax();
	}

    void df(const CudaMatrix* p, CudaMatrix* ap) {}
    ~SoftmaxFn() {}
};
