//
//  CudaOps.hpp
//  CudaMatrix
//
//  Created by Alessandro Sordoni on 2014-07-01.
//  Copyright (c) 2014 Alessandro Sordoni. All rights reserved.
//

#pragma once

#include <math.h>
#include <cuda.h>
#include <iostream>
#include "curand.h"
#include "cuda_runtime.h"
#include "device_launch_parameters.h"
#include "cublas_v2.h"
#include "cusparse_v2.h"
#include "Commons.hpp"

void CUDA_CALL(cudaError_t x);
void CUBLAS_CALL(cublasStatus_t x);
void CURAND_CALL(curandStatus_t x);

__global__ void GpuSetElement(float* a, size_t size);
__global__ void GpuProd(const long size, float* a, const float* b, const float* c);
__global__ void GpuDDAdd(size_t size, float* a, float* b, float f=1.0);
__global__ void GpuSDAdd(int aRows, const float* A, int bNz, int* Brow, int* Bcol, float* Bval, float f=1.0);
__global__ void GpuDSAdd(int aRows, float* A, int bNz, const int* Brow, const int* Bcol, const float* Bval, float f=1.0);
__global__ void GpuDDSMul(size_t aRows, size_t aCols,
                          size_t bRows, size_t bCols,
                          float* A, float* B, int* rows,
                          int* cols, float* valA);
__global__ void GpuMinMaxClip(const long size, float* A, float min, float max);

__global__ void GpuInplaceSigmoid(float* A, const long n);
__global__ void GpuInplaceSigmoidDerivative(const float* A, float* C, const long n);

__global__ void GpuInplaceSoftmax(size_t rows, size_t cols, float* A);
__global__ void GpuAssignCE(size_t rows,
                            size_t cols,
                            float* A,
                            float* E,
                            const int* target);
__global__ void GpuAssignCELoss(size_t rows,
                            size_t cols,
                            float* A,
                            int* T,
							float* loss);
__global__ void GpuAssignNCE(int nnoise,
                             int nz,
                             const float* valA,
                             const float* valP,
                             float* valE);
__global__ void GpuAssignNCELoss(int nnoise,
                             int nz,
                             const float* valA, // values of activation
                             const float* valP, // values of probabilities
                             float* e);
__global__ void GpuDotDSD(int aRows, int aCols,
                     float* aData, 
					 bool at,
					 bool bt,
					 int bNnz, 
					 int* bCooRow, 
                     int* bCooCol, 
					 float* bCooVal, 
					 float* cData,
					 int cRows);

__global__ void GpuMaxPooling(const long numChannels, 
	const long channelSize, 
	float* poolingAct,
	float* channelAct, 
	float* channelErrorMask);

__global__ void GpuDotStDD(float alpha,
    const float* val,
    const int* rowptr,
    const int* colptr,
    int nnz,
    const float* b,
    int numrows,
    int numcols,
    float* c,
    int numrowc);

__global__ void GpuAdagrad(
    float* gsq,
    float* gradient,
    float* weight,
    float learnRate,
	float mb,
	const size_t n);

__global__ void GpuAdadelta(
    float* gsq,
	float* xsq,
    float* gradient,
    float* weight,
    float learnRate,
	float mb,
    const size_t n);

__global__ void GpuDotSDD(float alpha,
    const float* val,
    const int* rowptr,
    const int* colptr,
    int nnz,
    const float* b,
    int numrows,
    int numcols,
	bool bTranspose,
    float* c,
    int numrowc);

__global__ void GpuDotSDS(int aNnz,
    const int* aRowptr,
    const int* aColptr,
    const float* aVal,
	bool aTranspose,
    int bRows,
    int bCols,
	const float* b,
    bool bTranspose,
    int* cRowPtr,
	int* cColPtr,
	float* cVal,
    int numrowc);

__global__ void GpuRescaleToRange(float *odata, const size_t size, float min, float max);
__global__ void GpuSetValue(float *odata, const float v, const size_t size);
__global__ void GpuTranspose(float *odata, const float *idata, int width, int height);

__global__ void GpuInplaceTanh(float* A, const long n);
__global__ void GpuInplaceTanhDerivative(const float* A, float* C, const long n);

__global__ void GpuInplaceRelu(float* A, const long n);
__global__ void GpuInplaceReluDerivative(const float* A, float* C, const long n);