//
//  GradientVisitor.cpp
//  Nnet
//
//  Created by Alessandro Sordoni on 2014-04-21.
//  Copyright (c) 2014 Alessandro Sordoni. All rights reserved.
//

#include "GradientVisitor.hpp"

GradientVisitor::GradientVisitor() : Visitor() {
	m_visitNodes = false; // only edges (i.e. weights)
}
GradientVisitor::~GradientVisitor() {
	destroy();
}

bool GradientVisitor::__upd_tiled(CudaMatrix* param, CudaMatrix* t_error, CudaMatrix* t_activation) {
	NOT_IMPLEMENTED;
}

Grads& GradientVisitor::ComputeGradients()
{
	return m_gradients; 
}

void GradientVisitor::__create_tiled(CudaMatrix* param, int expSize) { 
	if (m_tiledUpdateRow.find(param) != m_tiledUpdateRow.end())
		return;

	m_tiledUpdateRow.insert(std::make_pair(param, 0));
}

void GradientVisitor::CleanGradients() {
	for (Grads::iterator it = m_gradients.begin(); it != m_gradients.end(); it++) {
		it->second->Reset();
	}

	// Reset the row for tiled updates
	m_visitedWeights.clear();
}

void GradientVisitor::destroy() {
	for (Grads::iterator it = m_gradients.begin(); it != m_gradients.end(); it++) {
		delete it->second; it->second = NULL;
	}

	for (auto it = m_tiledUpdate.begin(); it != m_tiledUpdate.end(); it++) {	
		delete it->second.second; 
		delete it->second.first; 
	}

	m_visitedWeights.clear();
	m_gradients.clear();
}

CudaMatrix* GradientVisitor::GetOrCreateGradient(CudaMatrix* param, MatrixType gradientType) {
	auto gf = m_gradients.find(param);
	CudaMatrix* g = 0;
	if (gf == m_gradients.end()) {
		g = new CudaMatrix(param->GetRows(), param->GetCols(), gradientType);
		g->SetName("grad_" + param->GetName());
		m_gradients.insert(std::make_pair(param, g));
	} else
		g = gf->second;
	return g;
}

void GradientVisitor::Visit(DynamicWeight* w)
{
	// Current parameters
	CudaMatrix* param = w->m();
	Layer* child_hl = (Layer*) w->child();
	Layer* parent_hl = (Layer*) w->parent();

	// If this child layer is not errorated just return (its error is 0).
	// this condition is useful in BPTT and avoids to recompute dot of zero
	if (!child_hl->IsErrorated()) return;

#ifdef DEBUG
	std::cerr << "Updating: " << param->GetName() << " = " << (parent_hl->GetActivation())->GetName() << "^T"
		<< " * " << (child_hl->GetError())->GetName() << std::endl;
#endif
	CudaMatrix* dg;	
	/// Gradients are always dense..
	dg = GetOrCreateGradient(param, MatrixType::DENSE);
	CudaMatrix::Dot(*(parent_hl->GetActivation()), *(child_hl->GetError()), *dg, true, false);

#ifdef DEBUG
	std::cerr << "Gradient accumulated: " << std::endl;
	dg->Print();
#endif
}