//
//  gradient.hpp
//  nnet
//
//  Created by Alessandro Sordoni on 2014-03-25.
//  Copyright (c) 2014 Alessandro Sordoni. All rights reserved.
//

#pragma once

#include <unordered_map>
#include <algorithm>
#include <set>

#include "CudaMatrix.hpp"
#include "Visitor.hpp"
#include "NnetLib.hpp"

typedef std::unordered_map<CudaMatrix*, CudaMatrix*> Grads;
typedef std::unordered_map<CudaMatrix*, std::pair<float**, float**> > TiledUpdates;

/**
 ** Gradient visitor class, which goes through the graph
 ** and compute the gradients for each weight parameter 
 ** found in it.
 **/
class GradientVisitor : public Visitor {
protected:
	TiledUpdates m_tiledUpdate;
	std::unordered_map<CudaMatrix*, int> m_tiledUpdateRow;
	int m_batchSize;
    std::set<Weight*> m_visitedWeights;
    Grads m_gradients;
    
	CudaMatrix* GetOrCreateGradient(CudaMatrix* param, MatrixType gradientType);
    bool visited(Weight* w) {
        if (m_visitedWeights.find(w) != m_visitedWeights.end())
            return true;
        m_visitedWeights.insert(w);
        return false;
    }

    bool __upd_tiled(CudaMatrix* param, 
		CudaMatrix* t_error, 
		CudaMatrix* t_activation);
    
public:
    GradientVisitor();
	~GradientVisitor();
	
	Grads& gradients() { return m_gradients; } 
    virtual void CleanGradients();
    virtual void destroy();

	// Experimental method here.	
	void __create_tiled(CudaMatrix* param, int expSize);
	
    Grads& ComputeGradients();

    void post_visit() { m_visitedWeights.clear(); };
    void Visit(DynamicWeight* w);
};