//
//  rnn_entropy.cpp
//  Rnn
//
//  Created by Alessandro Sordoni on 3/10/2014.
//  Copyright (c) 2014 Alessandro Sordoni. All rights reserved.
//

#include <iostream>
#include <vector>
#include <unordered_map>
#include <string>
#include <algorithm>
#include <iomanip>
#include <iterator>
#include <thread>
#include <sstream>
#include <thread>
#include <mutex>

#include "RcgmTrainer.hpp"

int main(int argc, char* argv[])
{
	int threads = 1;
	std::string id = "unknown";
	std::string embeddings = "";
	int mb = 100;
	int epochs = 10;
	std::string output_path = "";
	std::string dataset_path = "";
	/// Reproducibility of results
	std::srand(1);


	if (argc == 1) {
		// Print help here!
		exit(1);
	}

	try {
		Config config = Config::ReadCommandLine(argc, argv);
		
		Vocabulary vocabulary;
		RandomGenerator random;
		
		std::string datasetPath = config.GetString("dataset");
		std::string trainDataset = datasetPath + ".train";
		std::string validDataset = datasetPath + ".valid";
		std::string testDataset = datasetPath + ".test";
		std::string dictDataset = datasetPath + ".dict";

		Vocabulary::Load(vocabulary, dictDataset);
		RandomGenerator::Create(random, vocabulary);

		// Set up the trainer
		RcgmTrainer trainer(config.GetString("output"));
		
		trainer.SetGenerator(&random);
		trainer.SetVocabulary(&vocabulary);

		int nThreads = config.GetInt("threads", 1);
		if (nThreads > 1)
			throw std::exception("Not possible to have more than 1 thread in GPU (for the moment).");
		trainer.SetNThreads(nThreads);
		
		trainer.SetEpochs(config.GetInt("epochs", 20));
		trainer.SetTau(config.GetInt("tau", 7));
		trainer.SetEpochSize(config.GetInt("epochSize", 1000000));

		trainer.SetTrainDataset(trainDataset);
		trainer.SetValidDataset(validDataset);
		trainer.SetTestDataset(testDataset);

		trainer.SetDelay(config.GetInt("delay", 5));
		trainer.SetActivation(config.GetString("act", "sigmoid"));
		
		int hidden = config.GetInt("hidden", 100);
		bool pretrained = config.GetBool("pretrained");
		if (pretrained)		
			trainer.InitializeRcgmWeights(config.GetString("pretrained"));
		else
			trainer.InitializeRcgmWeights(vocabulary.vocab_size, hidden, vocabulary.vocab_size);
		
		trainer.SetAlpha(config.GetFloat("alpha", 0.1));
		trainer.SetBeta(config.GetFloat("beta", 0.0000001));
		trainer.Train();

	} catch (std::exception e) {
		std::cerr << e.what() << std::endl;
		exit(-1);
	}

	return 0;
}