## Rescore the nbest list given a smoke scores tgt file
## and a cmn file.
##
## @author Alessandro Sordoni

import sys
import numpy

from tgt_reader import TgtReader
from lambda_reader import LambdaReader

print "#########################"
print "BLUEMAX RESCORING"
print "#########################"
print

if len(sys.argv) < 3:
	print >> sys.stderr, "[TGT_FILE] [CMN_FILE]"
	sys.exit(-1)

tgtR = TgtReader(sys.argv[1])
tgtR.includeFeatureValues = True

topK = 1
if len(sys.argv) > 3:
	topK = int(sys.argv[3]) # Nb of top K to write

lambdaR = LambdaReader(sys.argv[2])
lambdas = lambdaR.GetRecords()

numLambdas = len(lambdas.lambdas)
lambdaVector = numpy.array(lambdas.lambdas)

tgtRecords = tgtR.GetRecords()

firstCheck = False
bestHypothesis = []
bestScores = []
for head in tgtRecords:
	headScores = []

	if not firstCheck:
		for i, name in enumerate(head.featureNames[:numLambdas]):
			if name != lambdas.featureNames[i]:
				print >> sys.stderr, "Mismatch of feature names! check the ordering in the tgt and lambda file"
				sys.exit(-1)
		firstCheck = True
		
	for cand in head.rlist:
		featureVector = numpy.array(cand.featureValues[:numLambdas])
		# print lambdaVector
		headScores.append((cand.rtext, numpy.exp(numpy.dot(featureVector, lambdaVector))))
	
	headScores.sort(key=lambda x: x[1], reverse=True)
	toAppend = [x[0] for x in headScores[:topK]]
	scoresToAppend = [str(x[1]) for x in headScores[:topK]]
	
	bestScores.append("\t".join(scoresToAppend))
	bestHypothesis.append("\t".join(toAppend))

print "Length of best hypothesis %f" % len(bestHypothesis)
print "Writing best to %s " % str(sys.argv[2] + ".best" + str(topK) + ".r")

################## Writing everything to file
w = open(sys.argv[2] + ".best" + str(topK) + ".r.scores", "w")
w.write("\n".join(bestScores))
w.close()

w = open(sys.argv[2] + ".best" + str(topK) + ".r", "w")
w.write("\n".join(bestHypothesis))
w.close()
##################