# Produce the constant baseline as follows:
# Builds N constant baselines, where N is the number of references in the dev set. The i-th baseline consists of always outputting the i-th response of the dev set. Of these N constant baselines, select the dev sentence that has highest BLEU on the dev set. Evaluate that same baseline on the test set (using the dev-optimal sentence).
#
# @author Alessandro Sordoni

from bleu4 import Bleu4
import sys
import collections
import random
from permutation_test import PermutationTest
import numpy
import numpy as np
import copy

from tgt_reader import *
if len(sys.argv) < 2:
	print "Please specify a *development* tgt file"
	sys.exit(-1)

reader = TgtReader(sys.argv[1])
records = reader.GetRecords()
numRecords = len(records)

if numRecords == 0:
	print >> sys.stderr, "Are you sure that '%s' is a TGT file?" % sys.argv[1]
	sys.exit(-1)

# N^2
maxB4 = []
for i in records:
	### First true response
	cand = i.rtext[0]
	
	### Optimize on corpus bleu4
	bleu4 = Bleu4()
	for j in records:
		bleu4.UpdateMultiRefStats(j.rtext, cand)
	
	B4 = bleu4.GetB4()
	maxB4.append(B4)
	
	print "Done %s with bleu4 %f" % (i.rtext[0], B4)

maxIndex = max(enumerate(maxB4), key=lambda x:x[1])[0]

print "Writing best constant to %s " % str(sys.argv[1] + ".best.constant.r")
w = open(str(sys.argv[1] + ".best.constant.r"), "w")
for i in range(numRecords):
	w.write(records[maxIndex].rtext[0] + "\n")
w.close()