# Convert a file to IDS + UNK
#
# @author Alessandro Sordoni

import operator
import sys
import math

if len(sys.argv) < 2:
	print "USAGE: CUTOFF INPUT_DATA_PATH OUTPUT_DATA_PATH "
	sys.exit(0)


input_data = sys.argv[2]
output_data = input_data
if len(sys.argv) > 3:
	output_data = str(sys.argv[3])

dir_vocabulary = []
inv_vocabulary = collections.defaultdict(int)
idf_vocabulary = collections.defaultdict(int)

id = 0
lines = 0

try:
	for line in open(input_data, "r"):
		tokens = line.split()
		
		lines += 1
		if lines % 100000 == 0:
			print "Read ", lines
		
		alreadySeen = {}
		for token in tokens:
			token = token.strip()
			if len(token) == 0:
				continue
			
			inv_vocabulary[token] += 1
			if token not in alreadySeen:
				idf_vocabulary[token] += 1
				alreadySeen[token] = 1
except IOError as e:
    print "I/O error({0}): {1}".format(e.errno, e.strerror)

print " Read sentences = " , lines
print " Size of the vocabulary = " , len(inv_vocabulary)

listTokens = []
for key, value in inv_vocabulary.items():
	listTokens.append((key, value))
sortedTokens = sorted(listTokens, key=operator.itemgetter(1), reverse=True)

## Sort
# sorted_inv = sorted(inv_vocabulary.items(), key=operator.itemgetter(1), reverse=True)
# print " First token is = ", sorted_inv[0]
# cutoff = int(sys.argv[1])
# if cutoff == 0:
#	cutoff = len(sorted_inv)

cutoff = int(sys.argv[1])
if cutoff == 0:
	cutoff = len(sortedTokens)

sorted_inv = []
for token, value in sortedTokens:
	sorted_inv.append(token)

print " Size after cutoff = ", len(sorted_inv[:cutoff])
id_vocabulary = {}

id = 0
id_vocabulary["<s>"] = str(id)
idf_vocabulary["<s>"] = lines
id += 1
dir_vocabulary.append("<s>")
add_start_sentence = False
if "<s>" not in inv_vocabulary:
	print " Adding <s> token "
	add_start_sentence = True
	inv_vocabulary["<s>"] = lines

id_vocabulary["</s>"] = str(id)
idf_vocabulary["</s>"] = lines
id += 1
dir_vocabulary.append("</s>")
add_end_sentence = False
if "</s>" not in inv_vocabulary:
	print " Adding </s> token "
	add_end_sentence = True
	inv_vocabulary["</s>"] = lines

## Compute the unk_frequency
id_vocabulary["<unk>"] = str(id)
idf_vocabulary["<unk>"] = 0
id += 1
dir_vocabulary.append("<unk>")
if "<unk>" not in inv_vocabulary:
	print " Adding unk token "
	unk_frequency = 1
	for token in sorted_inv[cutoff+1:]:
		unk_frequency += 1
	inv_vocabulary["<unk>"] = unk_frequency
	
print " Assigning ids ..."
for token in sorted_inv[:cutoff]:
	if token == "<unk>" or token == "<s>" or token == "</s>":
		continue
	id_vocabulary[token] = str(id)
	dir_vocabulary.append(token)
	id += 1

print " Writing dictionary "
ow = open(output_data + ".dict", "w")
for token in dir_vocabulary:
	print >> ow, token + " " + str(inv_vocabulary[token]) + " " + str(math.log((lines+1)/(idf_vocabulary[token]+0.5)))
ow.close()

print " Writing output "
ow = open(output_data + ".train", "w")
for line in open(input_data, "r"):
	to_write = []
	sentences = line.split("\t")
	
	alreadyUnk = False
	line_to_write = []
	for sentence in sentences:
		sentence_to_write = []
		if add_start_sentence:
			sentence_to_write.append(id_vocabulary["<s>"])
		
		### Start string
		for token in sentence.split(" "):
			token = token.strip()
			if len(token) == 0:
				continue
			if token in id_vocabulary:
				sentence_to_write.append(id_vocabulary[token])
			else:
				sentence_to_write.append(id_vocabulary["<unk>"])
				if not alreadyUnk:
					idf_vocabulary["<unk>"] += 1
					alreadyUnk = True
		
		### End string
		if add_end_sentence:
			sentence_to_write.append(id_vocabulary["</s>"])
		
		line_to_write.append(" ".join(sentence_to_write))
	
	#### Here we conserve the structure of tab separated values!	
	print >> ow, '\t'.join(line_to_write)

ow.close()
print " Done "
