## Computes corpus bleu4 of nbest list.
##
## The tgt file is important in order to get the multireferences 
## and compute bleu4 statistics
##
## @author Alessandro Sordoni

from bleu4 import Bleu4
import sys
import collections
import random
from permutation_test import PermutationTest
import numpy
import numpy as np
import copy

def GetBestB4(refList, candList):
	""" Returns the candidate with the highest BLEU4+1 sentence level """
	b4s = []
	if len(candList) == 1:
		return candList[0]
	
	for x in candList:
		fb1 = Bleu4()
		fb1.UpdateMultiRefStats(refList, x)
		b4s.append(fb1.GetB4(0.1))
	
	maxIndex = max(enumerate(b4s), key=lambda x:x[1])
	return candList[maxIndex[0]]

from tgt_reader import *
if len(sys.argv) < 3:
	print "Please specify a tgt file and a candidate lists"
	sys.exit(-1)

reader = TgtReader(sys.argv[1])
records = reader.GetRecords()
numRecords = len(records)

if numRecords == 0:
	print >> sys.stderr, "Are you sure that '%s' is a TGT file?" % sys.argv[1]
	sys.exit(-1)

print >> sys.stderr, "Loaded ", numRecords, " references!"
print >> sys.stderr, "Loading candidate list : %s" % sys.argv[2]

firstSystem = open(sys.argv[2],"r").read().split("\n")
firstSystem = [x for x in firstSystem if len(x) != 0]

## Check consistency
if len(firstSystem) != numRecords:
	print >> sys.stderr, "Consistency error on external baseline: check the number of lines!"
	print len(firstSystem)
	sys.exit(-1)

globalMeanBleu4 = 0
referencesDone = 0
multiCount = 0

firstBleu = Bleu4()
for idx, record in enumerate(records):
	#### Compute top-K oracle bleu4
	fCandList = firstSystem[idx].split("\t")
	
	rlist = collections.OrderedDict()
	for i in record.rtext:
		rlist[i] = 1
	
	refList = rlist.keys()
	fCand = GetBestB4(record.rtext, fCandList)
	firstBleu.UpdateMultiRefStats(record.rtext, fCand)
	referencesDone += 1

firstCB4 = firstBleu.GetB4()
f = open("output.txt", "w")
print >> f, "Bleu4 for first system is {0}".format(firstCB4)
print >> sys.stderr, firstCB4
firstBleu.PrintStatistics()