# In order to convert a bl file into a tgt file
# we need to supply the bl file and a tgt file
# with the \REF for each bl line (in order)

import sys
import tgt_reader
import ir_reader
import bleu4

### Append Multireferences
appendMultireferences = False
### Done

reader = ir_reader.IRReader(sys.argv[1])
records = reader.GetRecords()

isPrelFile = (len(sys.argv) < 3)
if not isPrelFile:
	tgt = tgt_reader.TgtReader(sys.argv[2])
	trecords = tgt.GetRecords()

output = open(sys.argv[1] + ".tgt", "w")

if not isPrelFile and len(trecords) != len(records):
	sys.exit(-1)
	
### Write headers
done = 0
for index, value in enumerate(records):
	print >> output, value.mtext
	
	print value.ctext
	print value.mtext
	print
	
	multiRefs = []
	if isPrelFile:
		multiRefs = [x.rtext for x in value.rlist]
	else:
		### Grab it from the second tgt prel file
		multiRefs = trecords[index].rtext
	
	### Convert the multireferences
	for r in multiRefs:
		print >> output, "\\REF", r
	print multiRefs
		
	### Find this value in the IR file
	print >> output, "\\CTX", value.ctext
	
	output.write("@@HIR,WordPenalty,1-gramcount,1-grammatch,2-gramcount,2-grammatch,3-gramcount,3-grammatch,4-gramcount,4-grammatch,reflen\n")
	output.write("@@Q\n")
	
	if appendMultireferences:
		#!!!!!!!!!!!!! ATTENTION: Artificially append the multireferences as candidate hypothesis
		mean = 0.0
		for hyp in value.rlist:
			### Take the logs
			mean += hyp.score
		mean /= float(len(value.rlist))
		
		for rtext in multiRefs:
			tr = tgt_reader.TailRecord()
			tr.rtext = rtext
			### The unknown scores are mean values
			tr.score = mean
			tr.rid = ""
			value.rlist.append(tr)
	
	## multiRefSet = set(multiRefs)
	refsSuffStats = bleu4.SuffStats(multiRefs)
	
	for hyp in value.rlist:
		#### Strip away the exact matches
		## if hyp.rtext in multiRefSet:
		##	print "Stripping away ", hyp.rtext, hyp.rid
		##	continue
		candSuffStats = bleu4.SuffStats(hyp.rtext)
		
		b = bleu4.Bleu4()
		b.UpdateNgramMultiRefStatsFromSuffStats(refsSuffStats, candSuffStats)
		rMatches = [b.Get1GramCount(), b.Get1GramMatch(), b.Get2GramCount(), b.Get2GramMatch(),b.Get3GramCount(), b.Get3GramMatch(),b.Get4GramCount(), b.Get4GramMatch(), b.GetRefLen()]
		
		scores = [hyp.score, int(-b.Get1GramCount())]
		scores += rMatches
		
		print >> output, hyp.rtext
		print >> output, "@@V" + ",".join(map(str, scores))
	
	print >> output, "@@==="
	print >> output, "  -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-"
	print >> output, ""
	done += 1
	
output.close()