## Computes N*(N-1) Bleu4+1 agreement for
## three baselines - random, human and parrot - or from a 
## user input file (best generated reference).
## Example: python nnm1_bleu.py /cygdrive/d/Chitchat/ModelData/MtData/b5/smoke_1000best.mt.closest.OUT \\ /cygdrive/d/Chitchat/ModelData/MtData/b5/b5+bm/lambda_1000best.mt.closest.OUT.cmn.best1.r SAMPLING
## @author Alessandro Sordoni

import bleu4
import sys
import collections
import random
import numpy

from tgt_reader import *

if len(sys.argv) < 3:
	print "Please specify a TGT and a 1-best list"
	sys.exit(-1)

externalBest = [] ### For external baseline evaluation

reader = TgtReader(sys.argv[1])
records = reader.GetRecords()
numRecords = len(records)

humanEval = False
if sys.argv[2] == "HUMAN":
	humanEval = True
else:
	externalBest = open(sys.argv[2],"r").read().split("\n")

	## Check consistency
	if len(externalBest) != numRecords:
		print >> sys.stderr, "Consistency error on external baseline: check the number of lines!"
		sys.exit(-1)

sampling = False
if len(sys.argv) > 3:
	print "Sampling enabled"
	sampling = True
		
globalMeanBleu4 = 0
referencesDone = 0
counter = 0

avgBleu4 = []

samplingLength = 1
if sampling:
	samplingLength = 100

for i in range(samplingLength):
	b4sampling = bleu4.Bleu4()
	
	for index, record in enumerate(records):
		#### For each ref compute the N*(N-1) blue
		list = collections.OrderedDict()
	
		#### Filter out doubles (???? speak with Chris....)
		for rtext in record.rtext:
			list[rtext] = 1
		
		#### Now we have the multiref list
		list = list.keys() ### Flatten out dictionary
		if len(list) <= 1:
			continue
		
		if not humanEval:
			cand = externalBest[index]
		
		meanBleu4 = []
		
		if not sampling:
			for i in range(len(list)):
				if humanEval:
					cand = list[i]
			
				refList = [x for x in list if x != list[i]]
			
				b = bleu4.Bleu4()
				b.UpdateMultiRefStats(refList, cand)
				meanBleu4.append(b.GetB4(0.01))
			
			referencesDone += 1
			
			meanBleu4 = numpy.mean(numpy.array(meanBleu4, dtype="float32"))
			#if meanBleu4 > 0.2:
			#	print refList
			#	print cand
			#	print
			avgBleu4.append(meanBleu4)

		if sampling:
			index = random.randint(0, len(list) - 1)
			refList = [x for x in list if x != list[index]]
			
			if humanEval:
				cand = list[index]
			
			b4sampling.UpdateMultiRefStats(refList, cand)
	
	if sampling:
		avgBleu4.append(b4sampling.GetB4())

globalMeanBleu4 = numpy.mean(numpy.array(avgBleu4, dtype="float32"))

outputFile = sys.argv[2] + ".nnm1"
if sampling:
	outputFile += ".sampling"
	
if not humanEval:
	output = open(outputFile, "w")
	print >> output, "Effective references done {0}".format(referencesDone)
	print >> output, "Bleu4: {1}".format(type, float(globalMeanBleu4))
	output.close()
else:
	print "Effective references done {0}".format(referencesDone)
	print "Bleu4 is {1}".format(type, float(globalMeanBleu4))