# Reorder IR tgt file following MT order... 
# 
# @author Alessandro Sordoni

import sys
import tgt_reader
import ir_reader
import re
import bleu4
import numpy

if len(sys.argv) < 1:
	print "USAGE: TGT_MT IR_BL"
	sys.exit(-1)

## Load Tgt Records
irTgt = ir_reader.IRReader(sys.argv[2])
irRecords = irTgt.GetRecords()

tgt = tgt_reader.TgtReader(sys.argv[1])
records = tgt.GetRecords()

output = open(sys.argv[2] + ".tgt", "w")
idToRemove = open(sys.argv[3], "r").readlines()
ids = {}
for x in idToRemove:
	ids[x.strip()]=1

reorderIrRecords = []
for idx, value in enumerate(records):
	refMess = value.mtext
	refCtx = value.ctext
	
	if refCtx == "":
		print >> sys.stderr, "Must use a \CTX augmented file!"
		sys.exit(-1)
	
	done = False
	for irRecord in irRecords:
		mcopy = refMess.translate(None, "'`")
		ccopy = refCtx.translate(None, "'`")
		
		ircopy = irRecord.mtext.translate(None, "'`")
		irccopy = irRecord.ctext.translate(None, "'`")
		
		if mcopy == ircopy and ccopy == irccopy:
			done = True
			
			print >> output, refMess
			for r in value.rtext:
				print >> output, "\\REF", r
			
			print >> output, "\\CTX", refCtx
			output.write("@@HIR,WordPenalty,1-gramcount,1-grammatch,2-gramcount,2-grammatch,3-gramcount,3-grammatch,4-gramcount,4-grammatch,reflen\n")
			output.write("@@Q\n")
			
			refsSuffStats = bleu4.SuffStats(value.rtext)
			for hyp in irRecord.rlist:
				if hyp.rid in ids:
					print "skipping ", hyp.rid
					continue
				
				candSuffStats = bleu4.SuffStats(hyp.rtext)
				
				b = bleu4.Bleu4()
				b.UpdateNgramMultiRefStatsFromSuffStats(refsSuffStats, candSuffStats)
				rMatches = [b.Get1GramCount(), b.Get1GramMatch(), b.Get2GramCount(), b.Get2GramMatch(),b.Get3GramCount(), b.Get3GramMatch(),b.Get4GramCount(), b.Get4GramMatch(), b.GetRefLen()]
		
				scores = [hyp.score, int(-b.Get1GramCount())]
				scores += rMatches
		
				print >> output, hyp.rtext
				print >> output, "@@V" + ",".join(map(str, scores))
	
			print >> output, "@@==="
			print >> output, "  -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-"
			print >> output, ""

	if not done:
		print "M- ", refMess
		print "C- ", refCtx
		print "Not found"

output.close()