# A Reader for TGT files with additional capabilities 
# - i.e. stores \CTX tags
# and a Writer for TGT files. 
# WARNING: loading can take some time with big TGT files
# @author Alessandro Sordoni

import sys
import io
import re
import collections
import operator

def int_or_float(s):
    try:
        return int(s)
    except ValueError:
        return float(s)

class HeadRecord:
	def __init__(self):
		self.cid = ""
		self.ctext = ""
		self.mid = ""
		self.featureNames = []
		self.mtext = ""
		self.rtext = [] 	### List of multireferences
		self.rid = []		### List of multireferences ids (just 0 for tgt files)
		self.rlist = [] 	### List of candidate translations i.e. generated Rs

class TailRecord:
	def __init__(self):
		self.rid = ""
		self.featureValues = []
		self.score = 0
		self.rtext = ""
		
class TgtReader:
	def __init__(self, file):
		self.file = file
		self.includeFeatureValues = True
		self.currNumLine = 0
		self.numLines = 0
		self.linesRead = []
		self.currId = ""
		
	def GetRecords(self, keepOriginals=False):
		d = []
		
		currentMrHead = {}
		currentMrTails = []
		extract = False
		
		try:
			reader = open(self.file, "r")
		except IOError:
			print >> sys.stderr, "TgtReader : File not found!"
			sys.exit(-1)
		
		lines = reader.read()
		reader.close()
		
		decoded = []
		
		#### Format adapter (between utf-16 and uft-8)
		try:
			decodedU16 = lines.decode("utf-16")
		except UnicodeDecodeError:
			decodedU16 = ""
		
		decodedLines = decodedU16.split("\n")
		if len(decodedLines) <= 1:
			decodedLines = lines.split("\n")
		####
		
		print "TGT READER ===================="
		start = True
		for line in decodedLines:
			line = line.strip()
			
			if start:
				currentMrHead = HeadRecord()
				currentMrHead.ctext = ""
				currentMrHead.cid = ""
				currentMrHead.mtext = line.strip()
				currentMrHead.mid = ""
				currentMrHead.rlist = []
				start = False
				continue
			
			pos = line.find("\\REF")
			if pos != -1:
				currentMrHead.rtext.append(line[5:])
				### Do not now the RID
				currentMrHead.rid.append(0)
				continue
			
			pos = line.find("\\CTX")
			if pos != -1:
				currentMrHead.ctext = line[5:]
				currentMrHead.cid = 0
				continue
						
			pos = line.find("@@H")
			if pos != -1:
				if self.includeFeatureValues:
					fv = line[3:].split(",")
					currentMrHead.featureNames = [str(x) for x in fv]
				extract = True
				continue
			
			pos = line.find("@@Q")
			if pos != -1:
				extract = True
				continue
			
			pos = line.find("@@V")
			if pos != -1:
				if self.includeFeatureValues:
					### Extract feature values
					fv = line[3:].split(",")
					mr.featureValues = [int_or_float(x) for x in fv]
				extract = True
				continue
			
			pos = line.find("@@===")
			if pos != -1:
				extract = False
				continue
			
			if extract == True:
				mr = TailRecord()
				mr.rtext = line
				mr.score = 0
				mr.rid = ""
				currentMrTails.append(mr)
				extract = False
				continue
			
			if len(line) == 0:
				start = True
				key = currentMrHead.mtext
				
				currentMrHead.rlist = currentMrTails
				d.append(currentMrHead)
				currentMrTails = []
				extract = False

		key = currentMrHead.mtext
		if len(currentMrTails) != 0:
			## Append the last one
			d[-1].rlist = currentMrTails
			currentMrTails = []
		
		reader.close()
		self.numLines = len(d)
		
		print "Loaded records = %d" % self.numLines
		print "===================="
		
		if self.numLines == 0:
			print >> sys.stderr, "Are you sure that is a TGT file?"
			sys.exit(-1)
		
		return d
		
class TgtWriter:
	def __init__(self, file):
		self.file = file
		
	def WriteRecords(self, records):
		writer = open(self.file, "w")
		
		print "TGT WRITER ===================="
		for x in records:
			print >> writer, x.mtext
			for r in x.rtext:
				writer.write("\\REF " + r + "\r\n")
			
			writer.write("\\CTX " + x.ctext + "\r\n")
			writer.write("@@H" + ",".join(x.featureNames) + "\r\n")
			writer.write("@@Q\r\n")
			
			for r in x.rlist:
				writer.write(r.rtext + "\r\n")
				writer.write("@@V" + ",".join(map(str, r.featureValues)) + "\r\n")
			
			writer.write("@@===\r\n")
			writer.write("  -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-\r\n")
			writer.write("generate2 {}\r\n")
			writer.write("====================================================\r\n")
			writer.write("\r\n")
		
		print "Written %d" % len(records)
		print "===================="
		writer.close()