### Prepare the features for LAMBDA-RANK
### learning to rank method with surrogate loss

import tgt_reader
import sys
import bleu4
import collections

def ConvertToString(f):
	if f == 0:
		return "Bad"
	if f < .25:
		return "Fair"
	if f < .5:
		return "Good"
	if f < .75:
		return "Excellent"
	if f <= 1:
		return "Perfect"

if len(sys.argv) < 3:
	print "Not enough arguments! [FEATURE FILE] [LP-PARAMETER]"
	sys.exit(-1)

featureReader = tgt_reader.TgtReader(sys.argv[1])
featuresToInclude = sys.argv[2].split(",")

### Lp penalty
lpPenalty=1
outputFile=sys.argv[1] + "." + ".".join(featuresToInclude) + ".lambdamart"
if len(sys.argv) > 3:
	print "Set penalty parameter to = ", sys.argv[3]
	lpPenalty=float(sys.argv[3])
if len(sys.argv) > 4:
	print "Output file is = ", sys.argv[4]
	outputFile=sys.argv[4]
###

featureRecords = featureReader.GetRecords()

print "Record loaded = ", len(featureRecords)

output = open(outputFile, "w")

print "Including features ", featuresToInclude
dictFeaturesToInclude = set(featuresToInclude)
indexFeaturesToInclude = []

print >> output, "\t".join(["m:QueryId", "m:Url", "m:Rating"] + featuresToInclude)

found = 0
for index, featureRecord in enumerate(featureRecords):
	### The samples to add are all the positive examples that we have in prelsRecords + all the examples retrieved	
	idToRelevance = {}
	idToFeatures = []
	
	if len(indexFeaturesToInclude) == 0:
		indexFeaturesToInclude = [key for key, x in enumerate(featureRecord.featureNames) if x in dictFeaturesToInclude]
	
	#### Assign an id for each doc
	id = 0
	
	bscores = []
	for key, tail in enumerate(featureRecord.rlist):
		idToFeatures.append([str(abs(tail.featureValues[x])) for x in indexFeaturesToInclude])
		
		#### Surrogate for rating with Bleu4 score
		b = bleu4.Bleu4()
		b.UpdateMultiRefStats(featureRecord.rtext, tail.rtext)
		b4k = b.GetB4(0.01, lpPenalty)
		bscores.append((key, b4k))
	
	#### Reorder the scores take the top 20
	bscores = sorted(bscores,key=lambda x: x[1],reverse=True)
	length = min(len(bscores), 20)
	
	idToRelevance = dict(bscores[:length])
	
	### Print line now
	for key, tail in enumerate(featureRecord.rlist):
		relevanceValue = "Bad"
		if key in idToRelevance:
			relevanceValue = ConvertToString(idToRelevance[key])
			found += 1
		
		print >> output, "\t".join([str(index),tail.rtext,relevanceValue] + idToFeatures[key])

print "Produced relevance values ", str(found)

### Everything is done
output.close()