# Computes corpus bleu4 at K by randomization
# The tgt file is important in order to get the multireferences 
# and compute bleu4 statistics
#
# @author Alessandro Sordoni

from bleu4 import Bleu4
import sys
import collections
import random
from permutation_test import PermutationTest
import numpy
import numpy as np
import copy
from random import randint

from tgt_reader import *
if len(sys.argv) < 3:
	print "Please specify a tgt file and a candidate lists"
	sys.exit(-1)

reader = TgtReader(sys.argv[1])
records = reader.GetRecords()
numRecords = len(records)
K = 10

if numRecords == 0:
	print >> sys.stderr, "Are you sure that '%s' is a TGT file?" % sys.argv[1]
	sys.exit(-1)

numSystems = len(sys.argv[2:])
rankedSystems = []
for i in range(numSystems):
	print >> sys.stderr, "Loading candidate list : %s" % sys.argv[2+i]
	lines = open(sys.argv[2+i], "r").readlines()
	lines = [x.strip().split("\t") for x in lines if len(x) != 0]
	
	## Check consistency
	if len(lines) != numRecords:
		print >> sys.stderr, "Consistency error on external baseline: check the number of lines!"
		print len(lines)
		sys.exit(-1)
	
	rankedSystems.append(lines)

systemsBleu4 = []
for i in range(numSystems):
	systemsBleu4.append(Bleu4())

numTrials = K
trialsResults = numpy.zeros((numTrials, numSystems))

for trial in range(K):
	for recordIndex, record in enumerate(records):
		#### For each ref compute the N*(N-1) blue
		refList = collections.OrderedDict()
		for rtext in record.rtext:
			refList[rtext] = 1
		
		for system in range(numSystems):
			i = random.randint(0, K-1)
			sampledHyp = rankedSystems[system][recordIndex][i]
			systemsBleu4[system].UpdateMultiRefStats(refList, sampledHyp)
	
	for system in range(numSystems):
		b4 = systemsBleu4[system].GetB4()
		systemsBleu4[system].ResetStats()
		trialsResults[trial][system] = b4
	
	print "Done trial %d" % trial

print trialsResults
print "Results ", numpy.sum(trialsResults, axis=0)/float(numTrials)